/*
 * Filename Base64DecodeDataSource.java   Auther: Henrik Baastrup (c) 2001
 *
 * This software is licensed under the GNU LGPL (Lesser general Public License):
 *   http://www.gnu.org/copyleft/lesser.html
 *
 */

package com.hbaastrup.AutoReportGenerator;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import javax.activation.DataSource;

public class Base64DecodeDataSource implements DataSource
{
    static byte tbl[] = {
        'A','B','C','D','E','F','G','H',
        'I','J','K','L','M','N','O','P',
        'Q','R','S','T','U','V','W','X',
        'Y','Z','a','b','c','d','e','f',
        'g','h','i','j','k','l','m','n',
        'o','p','q','r','s','t','u','v',
        'w','x','y','z','0','1','2','3',
        '4','5','6','7','8','9','+','/'
    };
    byte[] bytes;
    int length;
    String contentType;
    String name;
    boolean selected = true;

    public Base64DecodeDataSource(byte[] bytes, String contentType, String name)
    {
        byte src[];
        int i, j;

        if (null == contentType) this.contentType = "application/octet-stream";
        else this.contentType = contentType;
        this.name = name;

        if (bytes!=null) length = 3*(bytes.length/4);
        else length = 0;
        if (length==0) {
            length = 1;
            this.bytes = new byte[1];
            this.bytes[0] = 0;
            return;
        }
        if (bytes[bytes.length-1]=='=') {
            if (bytes[bytes.length-2]=='=') length -= 2;
            else length--;
        }
        // Transform the array to binary data.
        for (i=0; i<bytes.length; i++) {
            if (bytes[i] >= 'A' && bytes[i] <='Z') bytes[i] -= 'A';
            else if (bytes[i] >= 'a' && bytes[i] <='z') bytes[i] += 26-'a';
            else if (bytes[i] >= '0' && bytes[i] <='9') bytes[i] += 52-'0';
            else if (bytes[i] == '+') bytes[i] = 62;
            else if (bytes[i] == '/') bytes[i] = 63;
            else bytes[i] = 0;
        }

        // Tramsform the 4x6 bits to 3x8 bits as required by base64
        j = 0;
        this.bytes = new byte[length];
        for (i=0; i<bytes.length; i+=4) {
            if (j >= length) continue;
            this.bytes[j] = (byte)((bytes[i] << 2) | ((bytes[i+1] >> 4) & 0x3)); j++;
            if (j >= length) continue;
            this.bytes[j] = (byte)((bytes[i+1] << 4) | ((bytes[i+2] >> 2) & 0xf)); j++;
            if (j >= length) continue;
            this.bytes[j] = (byte)((bytes[i+2] << 6) | bytes[i+3]); j++;
        }
    }

    public String getContentType() {return contentType;}

    public String getName() {return name;}

    public int getLength() {return length;}

    public InputStream getInputStream()
    {
        return new ByteArrayInputStream(bytes, 0, length);
    }//getInputStream

    public OutputStream getOutputStream() throws java.io.IOException
    {
        throw new FileNotFoundException();
    }//getOutputStream

    public void setSelected(boolean b) {selected = b;}
    public boolean getSelected() {return selected;}
    public boolean isSelected() {return selected;}
    public byte[] getBytes() {return bytes;}

    public static void main(String args[])
    {
        Base64EncodeDataSource encod;
        Base64DecodeDataSource decod;
        byte[] byts;
        int i;

        byts = new byte[10];
        for (i=1; i<=10; i++) byts[i-1] = (byte)(i+245);
	System.out.println("Start bytes:");
        for (i=0; i<10; i++) System.out.print(byts[i]+" ");
        System.out.println(" ");
        encod = new Base64EncodeDataSource(byts, null, "pippo");
        byts = encod.getBytes();
	System.out.println("Encoded bytes:");
        for (i=0; i<encod.getLength(); i++) System.out.print((char)byts[i]+" ");
        System.out.println(" ");
        decod = new Base64DecodeDataSource(byts, null, "poppo");
        byts = decod.getBytes();
	System.out.println("Decoded bytes:");
        for (i=0; i<decod.getLength(); i++) System.out.print(byts[i]+" ");
        System.out.println(" ");
    }//main
}//Base64DecodeDataSource

