/*
 * Filename ReportEditorServlet.java   Auther: Henrik Baastrup (c) 2001
 *
 * This software is licensed under the GNU LGPL (Lesser general Public License):
 *   http://www.gnu.org/copyleft/lesser.html
 */

package com.hbaastrup.AutoReportGenerator;

import java.io.IOException;
import java.sql.SQLException;
import java.util.Vector;
import java.text.SimpleDateFormat;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletContext;
import javax.servlet.RequestDispatcher;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class ReportEditorServlet  extends HttpServlet
{
    private java.sql.Connection dbConnection = null;
    private String forwardToJsp = "";
    int repNo = 0;
    private String date = null;
    private String originator = null;
    private String status = null;
    private String title = null;
    private String swVersion = null;
    private int priority = 0;
    private String description = null;
    private String changeRequired = null;
    private String environment = null;
    private String responsibleStaff = null;
    private String solution = null;
    private String decision = null;
    private String startDate = null;
    private String endDate = null;
    private String manpower = null;
    private Vector attachmentFields = new Vector(0);
    private String recipientAddr = null;
    private String senderAddr = null;
    private String subject = null;
    private String ccAddr = null;
    private boolean haveError = false;
    private String message = "";

    public void init(ServletConfig config) throws ServletException
    {
        String jdbcUrl = "";
        String jdbcDriver = "";
        String jdbcUser = "";
        String jdbcPassword = "";

        super.init(config);

        ControlServlet controlServlet = ControlServlet.getInstance();
        if (controlServlet==null) return;
        jdbcUrl = controlServlet.getJdbcUrl();
        jdbcDriver = controlServlet.getJdbcDriver();
        jdbcUser = controlServlet.getJdbcUser();
        jdbcPassword = controlServlet.getJdbcPassword();
 
        try {
            Class.forName(jdbcDriver);
            if ((dbConnection=java.sql.DriverManager.getConnection(jdbcUrl, jdbcUser, jdbcPassword))==null) {
                Log.log("ReportEditorServlet.init(): DriverManager.getConnection() returned null");
                return;
            }
            dbConnection.setAutoCommit(true);
        }
        catch (SecurityException ex) {
            Log.log("ReportEditorServlet.init(): "+ex);
            return;
        }
        catch (ClassNotFoundException ex) {
            Log.log("ReportEditorServlet.init(): "+ex);
            return;
        }
        catch (SQLException ex) {
            Log.log("ReportEditorServlet.init(): "+ex);
            return;
        }
    }//init

    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
    {
        java.io.PrintWriter out;
        String requested;
        String action = "";
        int reportType = 0;

        response.setContentType("text/html");
        out = response.getWriter();

        if ((requested=request.getParameter("action"))!=null) action = requested.trim();
        if ((requested=request.getParameter("repNo"))!=null) repNo = new Integer(requested.trim()).intValue();
        if ((requested=request.getParameter("date"))!=null) date = requested.trim();
        if ((requested=request.getParameter("originator"))!=null) originator = requested.trim();
        if ((requested=request.getParameter("status"))!=null) status = requested.trim();
        if ((requested=request.getParameter("title"))!=null) title = requested.trim();
        if ((requested=request.getParameter("swVersion"))!=null) swVersion = requested.trim();
        if ((requested=request.getParameter("priority"))!=null) {
            if (requested.equals("Routine")) priority = 1;
            else if (requested.equals("Urgent")) priority = 2;
            else if (requested.equals("Critical")) priority = 3;
            else priority = 0;
        }
        if ((requested=request.getParameter("description"))!=null) description = requested.trim();
        if ((requested=request.getParameter("changeRequired"))!=null) changeRequired = requested.trim();
        if ((requested=request.getParameter("environment"))!=null) environment = requested.trim();
        if ((requested=request.getParameter("responsibleStaff"))!=null) responsibleStaff = requested.trim();
        if ((requested=request.getParameter("decision"))!=null) decision = requested.trim();
        if ((requested=request.getParameter("startDate"))!=null) startDate = requested.trim();
        if ((requested=request.getParameter("endDate"))!=null) endDate = requested.trim();
        if ((requested=request.getParameter("manpower"))!=null) manpower = requested.trim();
        if ((requested=request.getParameter("solution"))!=null) solution = requested.trim();
        if ((requested=request.getParameter("ccAddr"))!=null) ccAddr = requested.trim();
        if ((requested=request.getParameter("senderAddr"))!=null) senderAddr = requested.trim();
        if ((requested=request.getParameter("ForwardJsp"))!=null) forwardToJsp = requested.trim();

        if (forwardToJsp.indexOf("spr")>=0) reportType = 1;
        else if (forwardToJsp.indexOf("scr")>=0) reportType = 2;
        else reportType = 0;

System.out.println("Get Forward to: "+forwardToJsp+ "  reportType="+reportType);
System.out.println("Action="+action);
        ServletContext sc = getServletContext();
        if (action.equals("load")) {
            resetAll(request);
            if (reportType==1) selectSpr(repNo);
            else if (reportType==2) selectScr(repNo);

            setPageParameters(request);
            RequestDispatcher rd = sc.getRequestDispatcher(forwardToJsp);
            rd.forward(request, response);
        }
        else if (action.equals("Update")) {
            if (reportType==1) updateSpr(repNo);
            else if (reportType==2) updateScr(repNo);
            if (haveError) outError(out);
            else outUpdated(out);
        }
        out.close();
    }//doGet

    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
    {
    }// doPost

    private void selectSpr(int sprNo)
    {
        java.sql.Statement dbStatement = null;
        java.sql.ResultSet rSet = null;
        java.sql.Timestamp tm;
        SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
        String qStr, str, subStr;
        int i, j;

        if (dbConnection==null) return;
        try {
            if ((dbStatement=dbConnection.createStatement())==null) {
                message = "Connection.createStatement() returned null";
                haveError = true;
                Log.log("ReportEditorServlet.selectSpr(): Connection.createStatement() returned null");
                return;
            }
            qStr = "select * from SoftwareProblemReport where sprNo="+sprNo;
            if ((rSet=dbStatement.executeQuery(qStr))==null) {
                message = "Statement.executeQuery() returned null";
                haveError = true;
                dbStatement.close();
                return;
            }
            if (rSet.next()) {
                if ((tm=rSet.getTimestamp("date"))!=null) date = sdf.format(tm);
                if ((str=rSet.getString("originator"))!=null) originator = str;
                if ((str=rSet.getString("status"))!=null) status = str;
                if ((str=rSet.getString("title"))!=null) title = str;
                if ((str=rSet.getString("swVersion"))!=null) swVersion = str;
                priority = rSet.getInt("priority");
                if ((str=rSet.getString("description"))!=null) description = str;
                if ((str=rSet.getString("environment"))!=null) environment = str;
                if ((str=rSet.getString("solution"))!=null) solution = str;
                if ((str=rSet.getString("decision"))!=null) decision = str;
                if ((str=rSet.getString("attachment"))!=null) {
                    j = 0;
                    do {
                        if ((i=str.indexOf(','))>=0) {
                            subStr = str.substring(j,i);
                            attachmentFields.add(subStr);
                            str = str.substring(i+1);
                            j = i + 1;
                        }
                    } while (i >= 0);
                }
                if ((str=rSet.getString("senderAddr"))!=null) senderAddr = str;
                if ((str=rSet.getString("ccAddr"))!=null) ccAddr = str;
            }
            if (rSet!=null) rSet.close();
            if (dbStatement!=null) dbStatement.close();
        }
        catch (SQLException ex) {
            message = "Exception: "+ex;
            haveError = true;
            Log.log("ReportEditorServlet.selectSpr(): "+ex);
        }
    }//selectSpr

    private void updateSpr(int sprNo)
    {
        java.sql.Statement dbStatement = null;
        String qStr;

        if (dbConnection==null) return;
        try {
            if ((dbStatement=dbConnection.createStatement())==null) {
                message = "Connection.createStatement() returned null";
                haveError = true;
                Log.log("ReportEditorServlet.updateSpr(): Connection.createStatement() returned null");
                return;
            }
            qStr = "update SoftwareProblemReport set date='"+controlDate(date)+"'";
            if (originator!=null) qStr += ",originator='"+controlString(originator.trim())+"'";
            if (status!=null) qStr += ",status='"+controlString(status.trim())+"'";
            if (title!=null) qStr += ",title='"+controlString(title.trim())+"'";
            if (swVersion!=null) qStr += ",swVersion='"+controlString(swVersion.trim())+"'";
            qStr += ",priority="+priority;
            if (description!=null) qStr += ",description='"+controlString(description.trim())+"'";
            if (environment!=null) qStr += ",environment='"+controlString(environment.trim())+"'";
            if (solution!=null) qStr += ",solution='"+controlString(solution.trim())+"'";
            if (decision!=null) qStr += ",decision='"+controlString(decision.trim())+"'";
            if (attachmentFields.size()>0) {
                qStr += ",attachment='";
                for (int i=0; i<attachmentFields.size(); i++) {
                    qStr += (String)attachmentFields.get(i)+",";
                }
                qStr += "'";
            }
            if (senderAddr!=null) qStr += ",senderAddr='"+controlString(senderAddr.trim())+"'";
            if (ccAddr!=null) qStr += ",ccAddr='"+controlString(ccAddr.trim())+"'";
            qStr += " where sprNo="+sprNo;

System.out.println(qStr);
            dbStatement.execute(qStr);

            if (dbStatement!=null) dbStatement.close();
        }
        catch (SQLException ex) {
            message = "Exception: "+ex;
            haveError = true;
System.out.println("Exception: "+ex);
            Log.log("ReportEditorServlet.updateSpr(): "+ex);
        }
     }//updateSpr

    private void selectScr(int scrNo)
    {
        java.sql.Statement dbStatement = null;
        java.sql.ResultSet rSet = null;
        java.sql.Timestamp tm;
        SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
        String qStr, str, subStr;
        int i, j;

        if (dbConnection==null) return;
        try {
            if ((dbStatement=dbConnection.createStatement())==null) {
                message = "Connection.createStatement() returned null";
                haveError = true;
                Log.log("ReportEditorServlet.selectScr(): Connection.createStatement() returned null");
                return;
            }
            qStr = "select * from SoftwareChangeRequest where scrNo="+scrNo;
            if ((rSet=dbStatement.executeQuery(qStr))==null) {
                message = "Statement.executeQuery() returned null";
                haveError = true;
                dbStatement.close();
                return;
            }
            if (rSet.next()) {
                if ((tm=rSet.getTimestamp("date"))!=null) date = sdf.format(tm);
                if ((str=rSet.getString("originator"))!=null) originator = str;
                if ((str=rSet.getString("status"))!=null) status = str;
                if ((str=rSet.getString("title"))!=null) title = str;
                if ((str=rSet.getString("swVersion"))!=null) swVersion = str;
                priority = rSet.getInt("priority");
                if ((str=rSet.getString("changeRequired"))!=null) changeRequired = str;
                if ((str=rSet.getString("responsibleStaff"))!=null) responsibleStaff = str;
                if ((tm=rSet.getTimestamp("startDate"))!=null) startDate = sdf.format(tm);
                if ((tm=rSet.getTimestamp("endDate"))!=null) endDate = sdf.format(tm);
                if ((str=rSet.getString("manpower"))!=null) manpower = str;
                if ((str=rSet.getString("attachment"))!=null) {
                    j = 0;
                    do {
                        if ((i=str.indexOf(','))>=0) {
                            subStr = str.substring(j,i-1);
                            attachmentFields.add(subStr);
                            str = str.substring(i+1);
                            j = i + 1;
                        }
                    } while (i >= 0);
                }
                if ((str=rSet.getString("senderAddr"))!=null) senderAddr = str;
                if ((str=rSet.getString("ccAddr"))!=null) ccAddr = str;
            }
            if (rSet!=null) rSet.close();
            if (dbStatement!=null) dbStatement.close();
        }
        catch (SQLException ex) {
            message = "Exception: "+ex;
            haveError = true;
            Log.log("ReportEditorServlet.selectScr(): "+ex);
        }
    }//selectScr

    private void updateScr(int scrNo)
    {
        java.sql.Statement dbStatement = null;
        String qStr;

        if (dbConnection==null) return;
        try {
            if ((dbStatement=dbConnection.createStatement())==null) {
                message = "Connection.createStatement() returned null";
                haveError = true;
                Log.log("ReportEditorServlet.updateScr(): Connection.createStatement() returned null");
                return;
            }
            qStr = "update SoftwareChangeRequest set date='"+controlDate(date)+"'";
            if (originator!=null) qStr += ",originator='"+controlString(originator.trim())+"'";
            if (status!=null) qStr += ",status='"+controlString(status.trim())+"'";
            if (title!=null) qStr += ",title='"+title.trim()+"'";
            if (swVersion!=null) qStr += ",swVersion='"+controlString(swVersion.trim())+"'";
            qStr += ",priority="+priority;
            if (changeRequired!=null) qStr += ",changeRequired='"+controlString(changeRequired.trim())+"'";
            if (environment!=null) qStr += ",environment='"+controlString(environment.trim())+"'";
            if (responsibleStaff!=null) qStr += ",responsibleStaff='"+controlString(responsibleStaff.trim())+"'";
            if (startDate!=null) qStr += ",startDate='"+controlDate(startDate.trim())+"'";
            if (endDate!=null) qStr += ",endDate='"+controlDate(endDate.trim())+"'";
            if (manpower!=null) qStr += ",manpower='"+controlString(manpower.trim())+"'";

            if (attachmentFields.size()>0) {
                qStr += ",attachment='";
                for (int i=0; i<attachmentFields.size(); i++) {
                    qStr += (String)attachmentFields.get(i)+",";
                }
                qStr += "'";
            }
            if (senderAddr!=null) qStr += ",senderAddr='"+controlString(senderAddr.trim())+"'";
            if (ccAddr!=null) qStr += ",ccAddr='"+controlString(ccAddr.trim())+"'";
            qStr += " where scrNo="+scrNo;

System.out.println(qStr);
            dbStatement.execute(qStr);

            if (dbStatement!=null) dbStatement.close();
        }
        catch (SQLException ex) {
            message = "Exception: "+ex;
            haveError = true;
System.out.println("Exception: "+ex);
            Log.log("ReportEditorServlet.updateScr(): "+ex);
        }
     }//updateSpr

    public void resetAll(HttpServletRequest req)
    {
        HttpSession session = req.getSession(false);
        if (req!=null) {
            for (int i=0; i<attachmentFields.size(); i++) {
                session.removeAttribute("attachmentFile"+i);
                session.removeAttribute("attachmentFileSel"+i);
            }
             session.removeAttribute("repNo");
             session.removeAttribute("originator");
             session.removeAttribute("status");
             session.removeAttribute("date");
             session.removeAttribute("title");
             session.removeAttribute("swVersion");
             session.removeAttribute("priority");
             session.removeAttribute("description");
             session.removeAttribute("changeRequired");
             session.removeAttribute("environment");
             session.removeAttribute("responsibleStaff");
             session.removeAttribute("decision");
             session.removeAttribute("startDate");
             session.removeAttribute("endDate");
             session.removeAttribute("manpower");
             session.removeAttribute("solution");
             session.removeAttribute("recipientAddr");
             session.removeAttribute("senderAddr");
             session.removeAttribute("ccAddr");
             session.removeAttribute("haveError");
        }
        attachmentFields = new Vector(0);
        date = null;
        originator = null;
        status = null;
        title = null;
        swVersion = null;
        priority = 0;
        description = null;
        changeRequired = null;
        environment = null;
        responsibleStaff = null;
        solution = null;
        decision = null;
        solution = null;
        startDate = null;
        endDate = null;
        manpower = null;
        recipientAddr = null;
        senderAddr = null;
        subject = null;
        ccAddr = null;
        haveError = false;
        message = "";
    }//resetAll

    private void setPageParameters(HttpServletRequest req)
    {
        HttpSession session = req.getSession(true);
        if (session==null) return;
        session.setAttribute("sessionType","DbUpdate");
        session.setAttribute("repNo",new Integer(repNo).toString());
        if (date!=null) session.setAttribute("date",date);
        if (originator!=null) session.setAttribute("originator",originator);
        if (status!=null) session.setAttribute("status",status);
        if (title!=null) session.setAttribute("title",title);
        if (swVersion!=null) session.setAttribute("swVersion",swVersion);
        switch (priority) {
            case 1:
                session.setAttribute("priority","Routine");
                break;
            case 2:
                session.setAttribute("priority","Urgent");
                break;
            case 3:
                session.setAttribute("priority","Critical");
                break;
            default:
                session.setAttribute("priority","Routine");
        }
        if (description!=null) session.setAttribute("description",description);
        if (changeRequired!=null) session.setAttribute("changeRequired",changeRequired);
        if (startDate!=null) session.setAttribute("startDate",startDate);
        if (endDate!=null) session.setAttribute("endDate",endDate);
        if (manpower!=null) session.setAttribute("manpower",manpower);
        if (environment!=null) session.setAttribute("environment",environment);
        if (responsibleStaff!=null) session.setAttribute("responsibleStaff",responsibleStaff);
        if (solution!=null) session.setAttribute("solution",solution);
        if (decision!=null) session.setAttribute("decision",decision);
        if (ccAddr!=null) session.setAttribute("ccAddr",ccAddr);
        if (senderAddr!=null) session.setAttribute("senderAddr",senderAddr);
        if (haveError) session.setAttribute("haveError","true");
        else session.setAttribute("haveError","false");
        if (message!=null) session.setAttribute("message",message);
        for (int i=0; i<attachmentFields.size(); i++) {
            session.setAttribute("attachmentFile"+i,(String)attachmentFields.get(i));
        }
        if (forwardToJsp!=null) session.setAttribute("ForwardJsp",forwardToJsp);
    }//setPageParameters

    private String controlDate(String str)
    {
        java.text.SimpleDateFormat dfIn;
        java.text.SimpleDateFormat dfOut;
        java.util.Date date;

        dfIn = new java.text.SimpleDateFormat("dd/MM/yyyy");
        dfOut = new java.text.SimpleDateFormat("yyyy-MM-dd");
        try {
            date = dfIn.parse(str);
        } catch (java.text.ParseException ex) {
            System.out.println("controlDate: "+ex);
            return "";
        }
        return dfOut.format(date);
    }//controlDate

    private String controlString(String str)
    {
        byte in[] = str.getBytes();
        byte out[] = new byte[in.length+in.length/2];
        int i, j;

        j = 0;
        for (i=0; i<in.length; i++) {
            switch (in[i]) {
                case ';':
                    out[j] = ' '; j++;
                    out[j] = in[i]; j++;
                    break;
                case '\'':
                    out[j] = '\''; j++;
                    out[j] = in[i]; j++;
                    break;
                default:
                    out[j] = in[i]; j++;
            }
        }

        return new String(out,0,j);
    }//controlString

    private void outError(java.io.PrintWriter out)
    {
        out.println("<html><head>");
        out.println("</head><body>");
        out.println("<h1>Error</h1>");
        out.println(message);
        out.println("</body></html>");
    }//outError

    private void outUpdated(java.io.PrintWriter out)
    {
        out.println("<html><head>");
        out.println("</head><body>");
        out.println("<h1>Updated</h1>");
        out.println("</body></html>");
    }//outUpdated
}//ReportEditorServlet
