/*
 * Filename ReportReader.java   Auther: Henrik Baastrup (c) 2001
 *
 * This software is licensed under the GNU LGPL (Lesser general Public License):
 *   http://www.gnu.org/copyleft/lesser.html
 *
 */

package com.hbaastrup.AutoReportGenerator;

import java.io.*;
import java.sql.*;
import java.text.*;
import java.util.*;
import javax.mail.*;
import javax.mail.internet.*;
import com.hbaastrup.AutoReportGenerator.dataLayer.*;
import com.hbaastrup.MiniXML.*;

public class ReportReader
{
    int projectId = 0;
    int repNo = 0;
    int pop3Port = 110;
    String pop3Host = "";
    String pop3User = "";
    String pop3Password = "";
    private MiniXML miniXml = new MiniXML();
    private SimpleDateFormat dateHourFormat = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
    private SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy");

    public void init()
    {
    }//init

    public void finish()
    {
    }//finish

    private String controlDate(String str)
    {
        java.text.SimpleDateFormat dfIn;
        java.text.SimpleDateFormat dfOut;
        java.util.Date date;

        dfIn = new java.text.SimpleDateFormat("dd/MM/yyyy");
        dfOut = new java.text.SimpleDateFormat("yyyy-MM-dd");
        try {
            date = dfIn.parse(str);
        } catch (java.text.ParseException ex) {
            System.out.println("controlDate: "+ex);
            return "";
        }
        return dfOut.format(date);
    }//controlDate

    private String controlString(String str)
    {
        byte in[] = str.getBytes();
        byte out[] = new byte[in.length+in.length/2];
        int i, j;

        j = 0;
        for (i=0; i<in.length; i++) {
            switch (in[i]) {
                case ';':
                    out[j] = ' '; j++;
                    out[j] = in[i]; j++;
                    break;
                case '\'':
                    out[j] = '\''; j++;
                    out[j] = in[i]; j++;
                    break;
                default:
                    out[j] = in[i]; j++;
            }
        }

        return new String(out,0,j);
    }//controlString

    private void insertSpr(com.hbaastrup.MiniXML.Element subTree)
    {
        int sprNo = 0;
        java.sql.Timestamp date = null;
        String originator = null;
        char status = ' ';
        String swTitle = null;
        String swVersion = null;
        int priority = 0;
        String problemDescription = null;
        String environment = null;
        String recommendedSolution = null;
        String srbDecision = null;
        String senderAddr = null;
        String ccAddr = null;
        com.hbaastrup.MiniXML.Element elm;
        SoftwareProblemReport softwareProblemReport = new SoftwareProblemReport();
        SequenceKeys sequenceKeys = new SequenceKeys("Software_Problem_Report");
        boolean createNewRec = true;

        if ((elm=subTree.searchFor("project-id"))!=null) {
            projectId = Integer.parseInt(elm.getValue());
        }
        else if ((elm=subTree.searchFor("spr-no"))!=null) {
            sprNo = Integer.parseInt(elm.getValue());
            repNo = sprNo;
        }
        if ((elm=subTree.searchFor("date"))!=null) {
            try {
                date = new Timestamp(dateHourFormat.parse(elm.getValue()).getTime());
            } catch (java.text.ParseException ex) {
                try {
                    date = new Timestamp(dateFormat.parse(elm.getValue()).getTime());
                } catch (java.text.ParseException pex) {
                    date = new Timestamp(System.currentTimeMillis());
                }
            }
        }
        if ((elm=subTree.searchFor("originator"))!=null) {
            originator = controlString(elm.getValue());
        }
        if ((elm=subTree.searchFor("status"))!=null) {
            status = controlString(elm.getValue()).charAt(0);
        }
        if ((elm=subTree.searchFor("title"))!=null) {
            swTitle = controlString(elm.getValue());
        }
        if ((elm=subTree.searchFor("swVersion"))!=null) {
            swVersion = controlString(elm.getValue());
        }
        if ((elm=subTree.searchFor("priority"))!=null) {
            priority = Integer.parseInt(elm.getValue());
        }
        if ((elm=subTree.searchFor("description"))!=null) {
            problemDescription = controlString(elm.getValue());
        }
        if ((elm=subTree.searchFor("environment"))!=null) {
            environment = controlString(elm.getValue());
        }
        if ((elm=subTree.searchFor("solution"))!=null) {
            recommendedSolution = controlString(elm.getValue());
        }
        if ((elm=subTree.searchFor("decision"))!=null) {
            srbDecision = controlString(elm.getValue());
        }
        if ((elm=subTree.searchFor("attachment"))!=null) {
        }
        if ((elm=subTree.searchFor("senderAddr"))!=null) {
            senderAddr = controlString(elm.getValue());
        }
        if ((elm=subTree.searchFor("ccAddr"))!=null) {
            ccAddr = controlString(elm.getValue());
        }
        
        if (projectId==0) projectId = Projects.DEFAULT_PROJECT_ID;
        try {
            if (sprNo==0) repNo = sprNo = sequenceKeys.getNextSequence();
            if (softwareProblemReport.findByPrimaryKey(projectId, sprNo)==null) createNewRec = true;
            else createNewRec = false;
        } catch (Exception ex) {
            System.out.println("insertSpr: "+ex);
        }
        if (date!=null) softwareProblemReport.setDate(date);
        if (originator!=null) softwareProblemReport.setOriginator(originator);
        if (status!=' ') softwareProblemReport.setStatus(status);
        if (swTitle!=null) softwareProblemReport.setSwTitle(swTitle);
        if (swVersion!=null) softwareProblemReport.setSwVersion(swVersion);
        if (priority!=0) softwareProblemReport.setPriority(priority);
        if (problemDescription!=null) softwareProblemReport.setProblemDescription(problemDescription);
        if (environment!=null) softwareProblemReport.setEnvironment(environment);
        if (recommendedSolution!=null) softwareProblemReport.setRecommendedSolution(recommendedSolution);
        if (srbDecision!=null) softwareProblemReport.setSrbDecision(srbDecision);
        if (senderAddr!=null) softwareProblemReport.setSenderAddr(senderAddr);
        if (ccAddr!=null) softwareProblemReport.setCcAddr(ccAddr);
        try {
            if (createNewRec) softwareProblemReport.create(projectId, sprNo);
            else softwareProblemReport.store();
        } catch (Exception ex) {
            System.out.println("insertSpr: "+ex);
        }
    }//insertSpr

    private void insertScr(com.hbaastrup.MiniXML.Element subTree)
    {
        int scrNo = 0;
        int sprNo = 0;
        java.sql.Timestamp date = null;
        String originator = null;
        char status = ' ';
        String swTitle = null;
        String swVersion = null;
        int priority = 0;
        String changeRequired = null;
        String responsibleStaff = null;
        java.sql.Timestamp startDate = null;
        java.sql.Timestamp endDate = null;
        int manpower = 0;
        String senderAddr = null;
        String ccAddr = null;
        com.hbaastrup.MiniXML.Element elm;
        SoftwareChangeRequest softwareChangeRequest = new SoftwareChangeRequest();
        SequenceKeys sequenceKeys = new SequenceKeys("Software_Change_Request");
        boolean createNewRec = true;

        if ((elm=subTree.searchFor("project-id"))!=null) {
            projectId = Integer.parseInt(elm.getValue());
        }
        else if ((elm=subTree.searchFor("scr-no"))!=null) {
            scrNo = Integer.parseInt(elm.getValue());
            repNo =sprNo;
        }
        else if ((elm=subTree.searchFor("spr-no"))!=null) {
            sprNo = Integer.parseInt(elm.getValue());
        }
        if ((elm=subTree.searchFor("date"))!=null) {
            try {
                date = new Timestamp(dateHourFormat.parse(elm.getValue()).getTime());
            } catch (java.text.ParseException ex) {
                try {
                    date = new Timestamp(dateFormat.parse(elm.getValue()).getTime());
                } catch (java.text.ParseException pex) {
                    date = new Timestamp(System.currentTimeMillis());
                }
            }
        }
        if ((elm=subTree.searchFor("originator"))!=null) {
            originator = controlString(elm.getValue());
        }
        if ((elm=subTree.searchFor("status"))!=null) {
            status = controlString(elm.getValue()).charAt(0);
        }
        if ((elm=subTree.searchFor("title"))!=null) {
            swTitle = controlString(elm.getValue());
        }
        if ((elm=subTree.searchFor("swVersion"))!=null) {
            swVersion = controlString(elm.getValue());
        }
        if ((elm=subTree.searchFor("priority"))!=null) {
            priority = Integer.parseInt(elm.getValue());
        }
        if ((elm=subTree.searchFor("change-required"))!=null) {
            changeRequired = controlString(elm.getValue());
        }
        if ((elm=subTree.searchFor("respopnsible-staff"))!=null) {
            responsibleStaff = controlString(elm.getValue());
        }
        if ((elm=subTree.searchFor("start-date"))!=null) {
            try {
                startDate = new Timestamp(dateHourFormat.parse(elm.getValue()).getTime());
            } catch (java.text.ParseException ex) {
                try {
                    startDate = new Timestamp(dateFormat.parse(elm.getValue()).getTime());
                } catch (java.text.ParseException pex) {
                    startDate = new Timestamp(System.currentTimeMillis());
                }
            }
        }
        if ((elm=subTree.searchFor("end-date"))!=null) {
            try {
                endDate = new Timestamp(dateHourFormat.parse(elm.getValue()).getTime());
            } catch (java.text.ParseException ex) {
                try {
                    endDate = new Timestamp(dateFormat.parse(elm.getValue()).getTime());
                } catch (java.text.ParseException pex) {
                    endDate = new Timestamp(System.currentTimeMillis());
                }
            }
        }
        if ((elm=subTree.searchFor("manpower"))!=null) {
            manpower = Integer.parseInt(elm.getValue());
        }
        if ((elm=subTree.searchFor("attachment"))!=null) {
        }
        if ((elm=subTree.searchFor("senderAddr"))!=null) {
            senderAddr = controlString(elm.getValue());
        }
        if ((elm=subTree.searchFor("ccAddr"))!=null) {
            ccAddr = controlString(elm.getValue());
        }

        if (projectId==0) projectId = Projects.DEFAULT_PROJECT_ID;
        try {
            if (scrNo==0) repNo = scrNo = sequenceKeys.getNextSequence();
            if (softwareChangeRequest.findByPrimaryKey(projectId, scrNo)==null) createNewRec = true;
            else createNewRec = false;
        } catch (Exception ex) {
            System.out.println("insertScr: "+ex);
        }
        if (sprNo!=0) softwareChangeRequest.setSprNo(sprNo);
        if (date!=null) softwareChangeRequest.setDate(date);
        if (originator!=null) softwareChangeRequest.setOriginator(originator);
        if (status!=' ') softwareChangeRequest.setStatus(status);
        if (swTitle!=null) softwareChangeRequest.setSwTitle(swTitle);
        if (swVersion!=null) softwareChangeRequest.setSwVersion(swVersion);
        if (priority!=0) softwareChangeRequest.setPriority(priority);
        if (changeRequired!=null) softwareChangeRequest.setChangeRequired(changeRequired);
        if (responsibleStaff!=null) softwareChangeRequest.setResponsibleStaff(responsibleStaff);
        if (startDate!=null) softwareChangeRequest.setStartDate(startDate);
        if (endDate!=null) softwareChangeRequest.setEndDate(endDate);
        if (manpower!=0) softwareChangeRequest.setManpower(manpower);
        if (senderAddr!=null) softwareChangeRequest.setSenderAddr(senderAddr);
        if (ccAddr!=null) softwareChangeRequest.setCcAddr(ccAddr);
        try {
            if (createNewRec) softwareChangeRequest.create(projectId, scrNo);
            else softwareChangeRequest.store();
        } catch (Exception ex) {
            System.out.println("insertScr: "+ex);
        }
    }//insertScr

    public void insertData()
    {
        com.hbaastrup.MiniXML.Element tree, subTree, elm;

        tree = miniXml.getTree();
miniXml.writeOutTree(System.out, tree);
        tree.initSearch();
        if ((subTree=tree.searchFor("SoftwareProblemReport"))!=null) insertSpr(subTree);
        else if ((subTree=tree.searchFor("SoftwareChangeRequest"))!=null) insertScr(subTree);
    }//insertData

    public void readData(BufferedReader in)
    {
        String line, inputData;
        try {
            inputData = "";
            while ((line=in.readLine())!=null) {
                inputData += line;
            }
            miniXml.init();
            miniXml.parse(inputData);
        }
        catch (IOException ex) {
            System.out.println("EXCEPTION: "+ex);
        }
    }//readData

    private void getPop3Messages()
    {
        Properties props = System.getProperties();
        javax.mail.Session session;
        javax.mail.Store store = null;
        javax.mail.Folder folder = null;
        javax.mail.Message messages[];
        javax.mail.Address fromAddresses[] = null;
        java.io.BufferedReader in;
        StringBuffer messgBuff = new StringBuffer("");
        String messgStr;
        String sprStr = null;
        String scrStr = null;
        String str;
        
        try {

            props.setProperty("mail.pop3.port", Integer.toString(pop3Port));
            session = Session.getDefaultInstance(props, null);
            store = session.getStore("pop3");
            store.connect(pop3Host, pop3User, pop3Password);

            folder = store.getFolder("INBOX");
            folder.open(Folder.READ_WRITE);
//            folder.open(Folder.READ_ONLY);
            //Get all messages
            messages = folder.getMessages();
            for (int i=0; i<messages.length; i++) {
                // Read a message
                if (messages[i].isMimeType("multipart/*")) {
                    Multipart multipart = (Multipart)messages[i].getContent();
                    for (int j=0; j<multipart.getCount(); j++) {
                        Part part = multipart.getBodyPart(j);
                        if (part.isMimeType("text/plain")) {
                            if (part.getFileName().equals("spr.xml")) sprStr = (String)part.getContent();
                            else if (part.getFileName().equals("scr.xml")) scrStr = (String)part.getContent();
                            else messgStr = (String)part.getContent();
                        }
                    }
                }
                else if (messages[i].isMimeType("text/plain")) {
                    if (messages[i].getFileName().equals("spr.xml")) sprStr = (String)messages[i].getContent();
                    else if (messages[i].getFileName().equals("scr.xml")) scrStr = (String)messages[i].getContent();
                    else messgStr = (String)messages[i].getContent();
                }
            }

            // Close connection 
            folder.close(true);
//            folder.close(false);
            store.close();
        } catch (Exception ex) {
            Log.log("getMessages", ex);
            try {
                folder.close(true);
                store.close();
            } catch (Exception exc) {
                Log.log("getMessages", exc);
            }
        }
    }//getMessages

    public static void main(String args[])
    {
        BufferedReader in = null;
        ReportReader rr = new ReportReader();

        if (args.length < 1) {
            System.out.println("Missing argument!");
            return;
        }

        try {
            in = new BufferedReader(new FileReader(args[0]));
        }
        catch (IOException ex) {
            System.out.println("EXCEPTION: "+ex);
            return;
        }

        rr.init();
        rr.readData(in);
        rr.insertData();
        rr.finish();
    }//main
}//ReportReader
