/*
 * Attachments.java     Auther: Henrik Baastrup (c) 2003
 *
 * This software is licensed under the GNU LGPL (Lesser general Public License):
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * Created on October 1, 2003, 6:34 PM
 */

package com.hbaastrup.AutoReportGenerator.dataLayer;
import java.sql.*;
import java.util.*;
import com.hbaastrup.util.*;
import com.hbaastrup.AutoReportGenerator.servlets.ReportGenerator;

public class Attachments {
    protected AttachmentsPK primaryKey = null;
    protected int reportNo = 0;
    protected int reportType = ReportGenerator.NULL_REPORT_TYPE;
    protected String fileName = " ";
    protected int size = 0;
    protected byte[] attachment = null;
    protected String contentType = "application/octet-stream";
    protected java.sql.Timestamp changed = null;
    protected java.sql.Timestamp timestamp = null;

    public Attachments()
    {
    }//Attachments
    
    public Attachments(int aProjectId, int aAttachmentId)
    {
        primaryKey = new AttachmentsPK(aProjectId, aAttachmentId);
    }//Attachments
    
    
    public boolean create(int aProjectId, int aAttachmentId) throws javax.naming.NamingException, java.sql.SQLException
    {
        primaryKey = new AttachmentsPK(aProjectId, aAttachmentId);
        return create();
    }//create
    
    public boolean create() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        Timestamp tm = new Timestamp(System.currentTimeMillis());
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "insert into Attachments ("
                   + "project_id, attachment_id, report_no, report_type, file_name, size, attachment, "
                   + "content_type, changed, timestamp "
                   + ") values (?,?,?,?,?,?,?,?,?,?)";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, primaryKey.projectId);
            ps.setInt(2, primaryKey.attachmentId);
            ps.setInt(3, reportNo);
            ps.setInt(4, reportType);
            ps.setString(5, fileName);
            ps.setInt(6, size);
            ps.setBytes(7, attachment);
            ps.setString(8, contentType);
            ps.setTimestamp(9, tm);
            ps.setTimestamp(10, tm);
            
            if (ps.executeUpdate() != 1) return false;
            
            timestamp = tm;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//create

    public boolean remove() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "delete from Attachments where "
                   + "project_id=? and attachment_id=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, primaryKey.projectId);
            ps.setInt(2, primaryKey.attachmentId);
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//remove

    public boolean removeByProjectId(int aProjectId) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "delete from Attachments where "
                   + "project_id=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, aProjectId);
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//removeByProjectId

    public boolean removeByProjectIdReportNoReportType(int aProjectId, int aReportNo, int aReportType) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "delete from Attachments where "
                   + "project_id=? and report_no=? and report_type=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, aProjectId);
            ps.setInt(2, aReportNo);
            ps.setInt(3, aReportType);
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//removeByProjectIdReportNoReportType

     public boolean store() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        Timestamp tm = new Timestamp(System.currentTimeMillis());
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "update Attachments set "
                   + "report_no=?, report_type=?, file_name=?, size=?, attachment=?, content_type=?, changed=? "
                   + "where project_id=? and attachment_id=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, reportNo);
            ps.setInt(2, reportType);
            ps.setString(3, fileName);
            ps.setInt(4, size);
            ps.setBytes(5, attachment);
            ps.setString(6,  contentType);
            ps.setTimestamp(7, tm);
            ps.setInt(8, primaryKey.projectId);
            ps.setInt(9, primaryKey.attachmentId);
            changed = tm;
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//store

   
    // Finder methods
    public AttachmentsPK findByPrimaryKey(int aProjectId, char aAttachmentId) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        ResultSet rs;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return null;
            sqlStr = "select * from Attachments "
                   + "where project_id=? and attachment_id=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return null;
            ps.setInt(1, aProjectId);
            ps.setInt(2, aAttachmentId);
            
            if ((rs=ps.executeQuery())==null) return null;
            if (!rs.next()) return null;
            primaryKey = new AttachmentsPK(aProjectId, aAttachmentId);
            reportNo = rs.getInt("report_no");
            reportType = (char)rs.getInt("report_type");
            fileName = rs.getString("file_name");
            size = rs.getInt("size");
            contentType = rs.getString("content_type");
            Blob blob = rs.getBlob("attachment");
            attachment = blob.getBytes(1, (int)blob.length());
            changed = rs.getTimestamp("changed");
            timestamp = rs.getTimestamp("timestamp");
            
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return primaryKey;
    }//findByPrimaryKey

    public Collection findByProjectIdReportNoReportType(int aProjectId, int aReportNo, int aReportType) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        ResultSet rs;
        Attachments attachments = null;
        String sqlStr;
        ArrayList retVal;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return null;
            sqlStr = "select * from Attachments "
                   + "where project_id=? and report_no=? and report_type=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return null;
            ps.setInt(1, aProjectId);
            ps.setInt(2, aReportNo);
            ps.setInt(3, aReportType);
            
            if ((rs=ps.executeQuery())==null) return null;
            retVal = new ArrayList();
            while (rs.next()) {
                attachments = new Attachments(aProjectId, rs.getInt("attachment_id"));
                attachments.reportNo = aReportNo;
                attachments.reportType = (char)rs.getInt("report_type");
                attachments.fileName = rs.getString("file_name");
                attachments.size = rs.getInt("size");
                attachments.contentType = rs.getString("content_type");
                Blob blob = rs.getBlob("attachment");
                attachments.attachment = blob.getBytes(1, (int)blob.length());
                attachments.changed = rs.getTimestamp("changed");
                attachments.timestamp = rs.getTimestamp("timestamp");
                retVal.add(attachments);
            }
            
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return retVal;
    }//findByProjectId

    
    // Public get methods for the primary key
    public AttachmentsPK getPrimaryKey() {return primaryKey;}
    public int getProjectId() {return primaryKey.projectId;}
    public int getAttachmentId() {return primaryKey.attachmentId;}
    
    // Methods to get/set the rest of the properties
    public int getReportNo() {return reportNo;}
    public void setReportNo(int aReportNo) {reportNo = aReportNo;}
 
    public int getReportType() {return reportType;}
    public void setReportType(int aType) {reportType = aType;}

    public String getFileName() {return fileName;}
    public void setFilename(String aFileName) {fileName = aFileName;}
    
    public String getContentType() {return contentType;}
    public void setContentType(String type) {contentType = type;}
    
    public void setAttachment(byte[] aAttachment) {attachment = new byte[aAttachment.length]; for (int i=0; i<aAttachment.length; i++) attachment[i] = aAttachment[i];}
    public byte[] getAttachment() {return attachment;}
    public byte[] getAttachmentBinary()
    {
        byte[] retVal = null;
        if (contentType.equals("application/base64")) {
            Base64DecodeDataSource decoder = new Base64DecodeDataSource(attachment, null, "attachement");
            retVal = decoder.getBytes();
        }
        else retVal = attachment;
        return retVal;
    }//getAttachmentBinary
    
    public java.sql.Timestamp getTimestamp() {return timestamp;}

}//Attachments
