/*
 * Projects.java     Auther: Henrik Baastrup (c) 2003
 *
 * This software is licensed under the GNU LGPL (Lesser general Public License):
 *   http://www.gnu.org/copyleft/lesser.html
 *
 *
 * Created on October 1, 2003, 4:40 PM
 */

package com.hbaastrup.AutoReportGenerator.dataLayer;
import java.sql.*;
import java.util.*;

public class Projects
{
    public static final int DEFAULT_PROJECT_ID = 1;
    
    protected int primaryKey = 0;
    protected String swTitle = " ";
    protected String swVersion = " ";
    protected String description = " ";
    protected java.sql.Timestamp changed = null;
    protected java.sql.Timestamp timestamp = null;
    
    public Projects()
    {
    }//Projects
    
    public Projects(int aPrimaryKey)
    {
        primaryKey = aPrimaryKey;
    }//Projects
    

    public boolean create(int aProjectId) throws javax.naming.NamingException, java.sql.SQLException
    {
        primaryKey = aProjectId;
        return create();
    }//create
    
    public boolean create() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        Timestamp tm = new Timestamp(System.currentTimeMillis());
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "insert into Projects ("
                   + "project_id, sw_title, sw_version, description, changed, timestamp "
                   + ") values (?,?,?,?,?,?)";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, primaryKey);
            ps.setString(2, swTitle);
            ps.setString(3, swVersion);
            ps.setString(4, description);
            ps.setTimestamp(5, tm);
            ps.setTimestamp(6, tm);
            
            if (ps.executeUpdate() != 1) return false;
            
            changed = tm;
            timestamp = tm;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//create

    public boolean remove() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "delete from Projects where "
                   + "project_id=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, primaryKey);
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//remove

    public boolean store() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        Timestamp tm = new Timestamp(System.currentTimeMillis());
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "update Projects set "
                   + "sw_title=?, sw_version=?, description=?, changed=? "
                   + "where project_id=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setString(1, swTitle);
            ps.setString(2, swVersion);
            ps.setString(3, description);
            ps.setTimestamp(4, tm);
            ps.setInt(5, primaryKey);
            changed = tm;
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//store


    // Finder methods
    public int findByPrimaryKey(int aProjectId) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        ResultSet rs;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return 0;
            sqlStr = "select * from Projects "
                   + "where project_id=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return 0;
            ps.setInt(1, aProjectId);
            
            if ((rs=ps.executeQuery())==null) return 0;
            if (!rs.next()) return 0;
            primaryKey = aProjectId;
            swTitle = rs.getString("sw_title");
            swVersion = rs.getString("sw_version");
            description = rs.getString("description");
            changed = rs.getTimestamp("changed");
            timestamp = rs.getTimestamp("timestamp");
            
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return primaryKey;
    }//findByPrimaryKey

    public Collection findAll() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        ResultSet rs;
        String sqlStr;
        Projects projectsRec;
        ArrayList retVal = new ArrayList();
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return null;
            sqlStr = "select * from Projects order by description";
            if ((ps=conn.prepareStatement(sqlStr))==null) return null;
            
            if ((rs=ps.executeQuery())==null) return null;
            while (rs.next()) {
                projectsRec = new Projects(rs.getInt("project_id"));
                projectsRec.swTitle = rs.getString("sw_title");
                projectsRec.swVersion = rs.getString("sw_version");
                projectsRec.description = rs.getString("description");
                projectsRec.changed = rs.getTimestamp("changed");
                projectsRec.timestamp = rs.getTimestamp("timestamp");
                retVal.add(projectsRec);
            }
            
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return retVal;
    }//findAll

    
    // Public get methods for the primary key
    public int getPrimaryKey() {return primaryKey;}
    public int getProjectId() {return primaryKey;}
    
    // Methods to get/set the rest of the properties
    public String getSwTitle() {return swTitle;}
    public void setSwTitle(String aTitle) {swTitle = aTitle;}
    
    public String getSwVersion() {return swVersion;}
    public void setSwVersion(String aVersion) {swVersion = aVersion;}
    
    public String getDescription() {return description;}
    public void setDescription(String aDescription) {description = aDescription;}
    
    public java.sql.Timestamp getChanged() {return changed;}
    public java.sql.Timestamp getTimestamp() {return timestamp;}
    
}//Projects
