/*
 * SequenceKeys.java        Auther: Henrik Baastrup (c) 2003
 *
 * This software is licensed under the GNU LGPL (Lesser general Public License):
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * Created on October 3, 2003, 11:46 AM
 */

package com.hbaastrup.AutoReportGenerator.dataLayer;
import java.sql.*;

public class SequenceKeys {
    protected SequenceKeysPK primaryKey = null;
    protected int sequence = 0;
    protected java.sql.Timestamp changed = null;
    protected java.sql.Timestamp timestamp = null;

    public SequenceKeys()
    {
    }//SequenceKeys
    
    public SequenceKeys(int aProjectId, String aTableName)
    {
        primaryKey = new SequenceKeysPK(aProjectId, aTableName);
    }//SequenceKeys

    
    public boolean create(int aProjectId, String aTableName) throws javax.naming.NamingException, java.sql.SQLException
    {
        primaryKey = new SequenceKeysPK(aProjectId, aTableName);;
        return create();
    }

    public boolean create() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        Timestamp tm = new Timestamp(System.currentTimeMillis());
        String sqlStr;

        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "insert into Sequence_Keys ("
                   + "project_id, table_name, sequence, changed, timestamp "
                   + ") values (?,?,?,?,?)";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, primaryKey.projectId);
            ps.setString(2, primaryKey.tableName);
            ps.setInt(3, sequence);
            ps.setTimestamp(4, tm);
            ps.setTimestamp(5, tm);
            
            if (ps.executeUpdate() != 1) return false;
            
            changed = tm;
            timestamp = tm;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//create
    
    public boolean removeByProjectId(int aProjectId) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "delete from Sequence_Keys where "
                   + "project_id=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, aProjectId);
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//removeByProjectId

    
    public synchronized int getNextSequence(int aProjectId, String aTableName) throws javax.naming.NamingException, java.sql.SQLException
    {
        primaryKey = new SequenceKeysPK(aProjectId, aTableName);;
        return getNextSequence();
    }
    
    public synchronized int getNextSequence() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        Timestamp tm = new Timestamp(System.currentTimeMillis());
        ResultSet rs;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return 0;
            sqlStr = "update Sequence_Keys set sequence=sequence+1, changed=? where project_id=? and table_name=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return 0;
            ps.setTimestamp(1,tm);
            ps.setInt(2, primaryKey.projectId);
            ps.setString(3, primaryKey.tableName);
            
            if (ps.executeUpdate() != 1) return 0;
            ps.close();
            changed = tm;
            
            sqlStr = "select sequence from Sequence_Keys where project_id=? and table_name=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return 0;
            ps.setInt(1, primaryKey.projectId);
            ps.setString(2, primaryKey.tableName);
            
            if ((rs=ps.executeQuery())==null) return 0;
            if (!rs.next()) return 0;
            sequence = rs.getInt("sequence");
            rs.close();
           
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return sequence;
    }//getNextSequence
    
    
    // Public get methods for the primary key
    public SequenceKeysPK getPrimaryKey() {return primaryKey;}
    public int getProjectId() {return primaryKey.projectId;}
    public String getTableName() {return primaryKey.tableName;}
    
    // Methods to get/set the rest of the properties
    public int getSequence() {return sequence;}
    public java.sql.Timestamp getChanged() {return changed;}
    public java.sql.Timestamp getTimestamp() {return timestamp;}

}//SequenceKeys
