/*
 * SoftwareChangeRequest.java   Auther: Henrik Baastrup (c) 2003
 *
 * This software is licensed under the GNU LGPL (Lesser general Public License):
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * Created on October 1, 2003, 6:02 PM
 */

package com.hbaastrup.AutoReportGenerator.dataLayer;
import java.sql.*;
import java.util.*;

public class SoftwareChangeRequest {
    protected SoftwareChangeRequestPK primaryKey = null;
    protected int smrNo = 0;
    protected java.sql.Timestamp date = new java.sql.Timestamp(0);
    protected String originator = " ";
    protected int state = 1;
    protected String swTitle = " ";
    protected String swVersion = " ";
    protected int priority = 0;
    protected String changeRequired = " ";
    protected String responsibleStaff = " ";
    protected java.sql.Timestamp startDate = new java.sql.Timestamp(0);
    protected java.sql.Timestamp endDate = new java.sql.Timestamp(0);
    protected String manpower = " ";
    protected String senderAddr = " ";
    protected String ccAddr = " ";
    protected java.sql.Timestamp changed = null;
    protected java.sql.Timestamp timestamp = null;
    
    public SoftwareChangeRequest()
    {
    }//SoftwareChangeRequest

    public SoftwareChangeRequest(int aProjectId, int aScrNo)
    {
        primaryKey = new SoftwareChangeRequestPK(aProjectId, aScrNo);
    }//SoftwareProblemReport

    public boolean create(int aProjectId, int aScrNo) throws javax.naming.NamingException, java.sql.SQLException
    {
        primaryKey = new SoftwareChangeRequestPK(aProjectId, aScrNo);
        return create();
    }//create
    
    public boolean create() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        Timestamp tm = new Timestamp(System.currentTimeMillis());
        String sqlStr;
        
        if (date==null) date = tm;
        if (startDate==null) startDate = tm;
        if (endDate==null) endDate = tm;
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "insert into Software_Change_Request ("
                   + "project_id, scr_no, smr_no, date, originator, state, sw_title, sw_version, "
                   + "priority, change_required, responsible_staff, start_date, "
                   + "end_date, manpower, sender_addr, cc_addr, "
                   + "changed, timestamp "
                   + ") values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, primaryKey.projectId);
            ps.setInt(2, primaryKey.scrNo);
            ps.setInt(3, smrNo);
            ps.setTimestamp(4, date);
            ps.setString(5, originator);
            ps.setInt(6, state);
            ps.setString(7, swTitle);
            ps.setString(8, swVersion);
            ps.setInt(9, priority);
            ps.setString(10, changeRequired);
            ps.setString(11, responsibleStaff);
            ps.setTimestamp(12, startDate);
            ps.setTimestamp(13, endDate);
            ps.setString(14, manpower);
            ps.setString(15, senderAddr);
            ps.setString(16, ccAddr);
            ps.setTimestamp(17, tm);
            ps.setTimestamp(18, tm);
            
            if (ps.executeUpdate() != 1) return false;
            
            changed = tm;
            timestamp = tm;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//create

    public boolean remove() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "delete from Software_Change_Request where "
                   + "project_id=? and scr_no=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, primaryKey.projectId);
            ps.setInt(2, primaryKey.scrNo);
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//remove

    public boolean removeByProjectId(int aProjectId) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "delete from Software_Change_Request where "
                   + "project_id=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, aProjectId);
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//removeByProjectId

    public boolean store() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        Timestamp tm = new Timestamp(System.currentTimeMillis());
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "update Software_Change_Request set "
                   + "smr_no=?, date=?, originator=?, state=?, sw_title=?, sw_version=?, "
                   + "priority=?, change_required=?, responsible_staff=?, start_date=?, "
                   + "end_date=?, manpower=?, sender_addr=?, cc_addr=?, changed=? "
                   + "where project_id=? and scr_no=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, smrNo);
            ps.setTimestamp(2, date);
            ps.setString(3, originator);
            ps.setInt(4, state);
            ps.setString(5, swTitle);
            ps.setString(6, swVersion);
            ps.setInt(7, priority);
            ps.setString(8, changeRequired);
            ps.setString(9, responsibleStaff);
            ps.setTimestamp(10, startDate);
            ps.setTimestamp(11, endDate);
            ps.setString(12, manpower);
            ps.setString(13, senderAddr);
            ps.setString(14, ccAddr);
            ps.setTimestamp(15, tm);
            ps.setInt(16, primaryKey.projectId);
            ps.setInt(17, primaryKey.scrNo);
            changed = tm;
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//store


    // Finder methods
    public SoftwareChangeRequestPK findByPrimaryKey(int aProjectId, int aScrNo) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        ResultSet rs;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return null;
            sqlStr = "select * from Software_Change_Request "
                   + "where project_id=? and scr_no=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return null;
            ps.setInt(1, aProjectId);
            ps.setInt(2, aScrNo);
            
            if ((rs=ps.executeQuery())==null) return null;
            if (!rs.next()) return null;
            primaryKey = new SoftwareChangeRequestPK(aProjectId, aScrNo);
            smrNo = (char)rs.getInt("smr_no");
            date = rs.getTimestamp("date");
            originator = rs.getString("originator");
            state = (char)rs.getInt("state");
            swTitle = rs.getString("sw_title");
            swVersion = rs.getString("sw_version");
            priority = rs.getInt("priority");
            changeRequired = rs.getString("change_required");
            responsibleStaff = rs.getString("responsible_staff");
            startDate = rs.getTimestamp("start_date");
            endDate = rs.getTimestamp("end_date");
            manpower = rs.getString("manpower");
            senderAddr = rs.getString("sender_addr");
            ccAddr = rs.getString("cc_addr");
            changed = rs.getTimestamp("changed");
            timestamp = rs.getTimestamp("timestamp");
            
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return primaryKey;
    }//findByPrimaryKey

    public Collection findByProjectId(int aProjectId) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        ResultSet rs;
        SoftwareChangeRequest softwareChangeRequest = null;
        String sqlStr;
        ArrayList retVal;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return null;
            sqlStr = "select * from Software_Change_Request "
                   + "where project_id=? order by scr_no desc";
            if ((ps=conn.prepareStatement(sqlStr))==null) return null;
            ps.setInt(1, aProjectId);
            
            if ((rs=ps.executeQuery())==null) return null;
            retVal = new ArrayList();
            while (rs.next()) {
                softwareChangeRequest = new SoftwareChangeRequest(aProjectId, rs.getInt("scr_no"));
                softwareChangeRequest.smrNo = rs.getInt("smr_no");
                softwareChangeRequest.date = rs.getTimestamp("date");
                softwareChangeRequest.originator = rs.getString("originator");
                softwareChangeRequest.state = (char)rs.getInt("state");
                softwareChangeRequest.swTitle = rs.getString("sw_title");
                softwareChangeRequest.swVersion = rs.getString("sw_version");
                softwareChangeRequest.priority = rs.getInt("priority");
                softwareChangeRequest.changeRequired = rs.getString("change_required");
                softwareChangeRequest.responsibleStaff = rs.getString("responsible_staff");
                softwareChangeRequest.startDate = rs.getTimestamp("start_date");
                softwareChangeRequest.endDate = rs.getTimestamp("end_date");
                softwareChangeRequest.manpower = rs.getString("manpower");
                softwareChangeRequest.senderAddr = rs.getString("sender_addr");
                softwareChangeRequest.ccAddr = rs.getString("cc_addr");
                softwareChangeRequest.changed = rs.getTimestamp("changed");
                softwareChangeRequest.timestamp = rs.getTimestamp("timestamp");
                retVal.add(softwareChangeRequest);
            }
            
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return retVal;
    }//findByProjectId

    public Collection findByProjectIdSmrNo(int aProjectId, int aSmrNo) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        ResultSet rs;
        SoftwareChangeRequest softwareChangeRequest = null;
        String sqlStr;
        ArrayList retVal;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return null;
            sqlStr = "select * from Software_Change_Request "
                   + "where project_id=? and Smr_No=? order by scr_no";
            if ((ps=conn.prepareStatement(sqlStr))==null) return null;
            ps.setInt(1, aProjectId);
            ps.setInt(2, aSmrNo);
            
            if ((rs=ps.executeQuery())==null) return null;
            retVal = new ArrayList();
            while (rs.next()) {
                softwareChangeRequest = new SoftwareChangeRequest(aProjectId, aSmrNo);
                softwareChangeRequest.smrNo = rs.getInt("smr_no");
                softwareChangeRequest.date = rs.getTimestamp("date");
                softwareChangeRequest.originator = rs.getString("originator");
                softwareChangeRequest.state = (char)rs.getInt("state");
                softwareChangeRequest.swTitle = rs.getString("sw_title");
                softwareChangeRequest.swVersion = rs.getString("sw_version");
                softwareChangeRequest.priority = rs.getInt("priority");
                softwareChangeRequest.changeRequired = rs.getString("change_required");
                softwareChangeRequest.responsibleStaff = rs.getString("responsible_staff");
                softwareChangeRequest.startDate = rs.getTimestamp("start_date");
                softwareChangeRequest.endDate = rs.getTimestamp("end_date");
                softwareChangeRequest.manpower = rs.getString("manpower");
                softwareChangeRequest.senderAddr = rs.getString("sender_addr");
                softwareChangeRequest.ccAddr = rs.getString("cc_addr");
                softwareChangeRequest.changed = rs.getTimestamp("changed");
                softwareChangeRequest.timestamp = rs.getTimestamp("timestamp");
                retVal.add(softwareChangeRequest);
            }
            
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return retVal;
    }//findByProjectIdSmrNo


    // Public get methods for the primary key
    public SoftwareChangeRequestPK getPrimaryKey() {return primaryKey;}
    public int getProjectId() {return primaryKey.projectId;}
    public int getScrNo() {return primaryKey.scrNo;}
    
    // Methods to get/set the rest of the properties
    public int getSmrNo() {return smrNo;}
    public void setSmrNo(int no) {smrNo = no;}
    
    public java.sql.Timestamp getDate() {return date;}
    public void setDate(java.sql.Timestamp aDate) {date = aDate;}

    public String getOriginator() {return originator;}
    public void setOriginator(String aOriginator) {originator = aOriginator;}
    
    public int getState() {return state;}
    public void setState(int aState) {state = aState;}
    
    public String getSwTitle() {return swTitle;}
    public void setSwTitle(String aTitle) {swTitle = aTitle;}
    
    public String getSwVersion() {return swVersion;}
    public void setSwVersion(String aVersion) {swVersion = aVersion;}
    
    public int getPriority() {return priority;}
    public void setPriority(int aPriority) {priority = aPriority;}
    
    public String getChangeRequired() {return changeRequired;}
    public void setChangeRequired(String aChange) {changeRequired= aChange;}
    
    public String getResponsibleStaff() {return responsibleStaff;}
    public void setResponsibleStaff(String aStaff) {responsibleStaff = aStaff;}
    
    public java.sql.Timestamp getStartDate() {return startDate;}
    public void setStartDate(java.sql.Timestamp aDate) {startDate = aDate;}
    
    public java.sql.Timestamp getEndDate() {return endDate;}
    public void setEndDate(java.sql.Timestamp aDate) {endDate = aDate;}
    
    public String getManpower() {return manpower;}
    public void setManpower(String aManpower) {manpower = aManpower;}
    
    public String getSenderAddr() {return senderAddr;}
    public void setSenderAddr(String aAddr) {senderAddr = aAddr;}
    
    public String getCcAddr() {return ccAddr;}
    public void setCcAddr(String aAddr) {ccAddr = aAddr;}

    public java.sql.Timestamp getChanged() {return changed;}
    public java.sql.Timestamp getTimestamp() {return timestamp;}
    
}//SoftwareChangeRequest
