/*
 * SoftwareModificationReport.java   Auther: Henrik Baastrup (c) 2003
 *
 * This software is licensed under the GNU LGPL (Lesser general Public License):
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * Created on October 1, 2003, 6:02 PM
 */

package com.hbaastrup.AutoReportGenerator.dataLayer;
import java.sql.*;
import java.util.*;

public class SoftwareModificationReport {
    protected SoftwareModificationReportPK primaryKey = null;
    protected java.sql.Timestamp date = new java.sql.Timestamp(0);
    protected String originator = " ";
    protected String swTitle = " ";
    protected String swVersion = " ";
    protected String changeImplemented = " ";
    protected java.sql.Timestamp startDate = new java.sql.Timestamp(0);
    protected java.sql.Timestamp endDate = new java.sql.Timestamp(0);
    protected String manpower = " ";
    protected String senderAddr = " ";
    protected String ccAddr = " ";
    protected java.sql.Timestamp changed = null;
    protected java.sql.Timestamp timestamp = null;
    
    public SoftwareModificationReport()
    {
    }//SoftwareChangeRequest

    public SoftwareModificationReport(int aProjectId, int aSmrNo)
    {
        primaryKey = new SoftwareModificationReportPK(aProjectId, aSmrNo);
    }//SoftwareProblemReport

    public boolean create(int aProjectId, int aSmrNo) throws javax.naming.NamingException, java.sql.SQLException
    {
        primaryKey = new SoftwareModificationReportPK(aProjectId, aSmrNo);
        return create();
    }//create
    
    public boolean create() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        Timestamp tm = new Timestamp(System.currentTimeMillis());
        String sqlStr;
        
        if (date==null) date = tm;
        if (startDate==null) startDate = tm;
        if (endDate==null) endDate = tm;
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "insert into Software_Modification_Report ("
                   + "project_id, smr_no, date, originator, sw_title, sw_version, "
                   + "change_implemented, start_date, "
                   + "end_date, manpower, sender_addr, cc_addr, "
                   + "changed, timestamp "
                   + ") values (?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, primaryKey.projectId);
            ps.setInt(2, primaryKey.smrNo);
            ps.setTimestamp(3, date);
            ps.setString(4, originator);
            ps.setString(5, swTitle);
            ps.setString(6, swVersion);
            ps.setString(7, changeImplemented);
            ps.setTimestamp(8, startDate);
            ps.setTimestamp(9, endDate);
            ps.setString(10, manpower);
            ps.setString(11, senderAddr);
            ps.setString(12, ccAddr);
            ps.setTimestamp(13, tm);
            ps.setTimestamp(14, tm);
            
            if (ps.executeUpdate() != 1) return false;
            
            changed = tm;
            timestamp = tm;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//create

    public boolean remove() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "delete from Software_Modification_Report where "
                   + "project_id=? and smr_no=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, primaryKey.projectId);
            ps.setInt(2, primaryKey.smrNo);
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//remove

    public boolean removeByProjectId(int aProjectId) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "delete from Software_Modification_Report where "
                   + "project_id=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, aProjectId);
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//removeByProjectId

    public boolean store() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        Timestamp tm = new Timestamp(System.currentTimeMillis());
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "update Software_Modification_Report set "
                   + "date=?, originator=?, sw_title=?, sw_version=?, "
                   + "change_implemented=?, start_date=?, "
                   + "end_date=?, manpower=?, sender_addr=?, cc_addr=?, changed=? "
                   + "where project_id=? and smr_no=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setTimestamp(1, date);
            ps.setString(2, originator);
            ps.setString(3, swTitle);
            ps.setString(4, swVersion);
            ps.setString(5, changeImplemented);
            ps.setTimestamp(6, startDate);
            ps.setTimestamp(7, endDate);
            ps.setString(8, manpower);
            ps.setString(9, senderAddr);
            ps.setString(10, ccAddr);
            ps.setTimestamp(11, tm);
            ps.setInt(12, primaryKey.projectId);
            ps.setInt(13, primaryKey.smrNo);
            changed = tm;
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//store


    // Finder methods
    public SoftwareModificationReportPK findByPrimaryKey(int aProjectId, int aSmrNo) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        ResultSet rs;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return null;
            sqlStr = "select * from Software_Modification_Report "
                   + "where project_id=? and smr_no=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return null;
            ps.setInt(1, aProjectId);
            ps.setInt(2, aSmrNo);
            
            if ((rs=ps.executeQuery())==null) return null;
            if (!rs.next()) return null;
            primaryKey = new SoftwareModificationReportPK(aProjectId, aSmrNo);
            date = rs.getTimestamp("date");
            originator = rs.getString("originator");
            swTitle = rs.getString("sw_title");
            swVersion = rs.getString("sw_version");
            changeImplemented = rs.getString("change_implemented");
            startDate = rs.getTimestamp("start_date");
            endDate = rs.getTimestamp("end_date");
            manpower = rs.getString("manpower");
            senderAddr = rs.getString("sender_addr");
            ccAddr = rs.getString("cc_addr");
            changed = rs.getTimestamp("changed");
            timestamp = rs.getTimestamp("timestamp");
            
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return primaryKey;
    }//findByPrimaryKey

    public Collection findByProjectId(int aProjectId) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        ResultSet rs;
        SoftwareModificationReport softwareModificationReport = null;
        String sqlStr;
        ArrayList retVal;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return null;
            sqlStr = "select * from Software_Modification_Report "
                   + "where project_id=? order by smr_no desc";
            if ((ps=conn.prepareStatement(sqlStr))==null) return null;
            ps.setInt(1, aProjectId);
            
            if ((rs=ps.executeQuery())==null) return null;
            retVal = new ArrayList();
            while (rs.next()) {
                softwareModificationReport = new SoftwareModificationReport(aProjectId, rs.getInt("smr_no"));
                softwareModificationReport.date = rs.getTimestamp("date");
                softwareModificationReport.originator = rs.getString("originator");
                softwareModificationReport.swTitle = rs.getString("sw_title");
                softwareModificationReport.swVersion = rs.getString("sw_version");
                softwareModificationReport.changeImplemented = rs.getString("change_implemented");
                softwareModificationReport.startDate = rs.getTimestamp("start_date");
                softwareModificationReport.endDate = rs.getTimestamp("end_date");
                softwareModificationReport.manpower = rs.getString("manpower");
                softwareModificationReport.senderAddr = rs.getString("sender_addr");
                softwareModificationReport.ccAddr = rs.getString("cc_addr");
                softwareModificationReport.changed = rs.getTimestamp("changed");
                softwareModificationReport.timestamp = rs.getTimestamp("timestamp");
                retVal.add(softwareModificationReport);
            }
            
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return retVal;
    }//findByProjectId


    // Public get methods for the primary key
    public SoftwareModificationReportPK getPrimaryKey() {return primaryKey;}
    public int getProjectId() {return primaryKey.projectId;}
    public int getSmrNo() {return primaryKey.smrNo;}
    
    // Methods to get/set the rest of the properties
    public java.sql.Timestamp getDate() {return date;}
    public void setDate(java.sql.Timestamp aDate) {date = aDate;}

    public String getOriginator() {return originator;}
    public void setOriginator(String aOriginator) {originator = aOriginator;}
    
    public String getSwTitle() {return swTitle;}
    public void setSwTitle(String aTitle) {swTitle = aTitle;}
    
    public String getSwVersion() {return swVersion;}
    public void setSwVersion(String aVersion) {swVersion = aVersion;}
    
    public String getChangeImplemented() {return changeImplemented;}
    public void setChangeImplemented(String aChange) {changeImplemented= aChange;}
    
    public java.sql.Timestamp getStartDate() {return startDate;}
    public void setStartDate(java.sql.Timestamp aDate) {startDate = aDate;}
    
    public java.sql.Timestamp getEndDate() {return endDate;}
    public void setEndDate(java.sql.Timestamp aDate) {endDate = aDate;}
    
    public String getManpower() {return manpower;}
    public void setManpower(String aManpower) {manpower = aManpower;}
    
    public String getSenderAddr() {return senderAddr;}
    public void setSenderAddr(String aAddr) {senderAddr = aAddr;}
    
    public String getCcAddr() {return ccAddr;}
    public void setCcAddr(String aAddr) {ccAddr = aAddr;}

    public java.sql.Timestamp getChanged() {return changed;}
    public java.sql.Timestamp getTimestamp() {return timestamp;}
    
}//SoftwareChangeRequest
