/*
 * SoftwareProblemReport.java   Auther: Henrik Baastrup (c) 2003
 *
 * This software is licensed under the GNU LGPL (Lesser general Public License):
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * Created on October 1, 2003, 5:07 PM
 */

package com.hbaastrup.AutoReportGenerator.dataLayer;
import java.sql.*;
import java.util.*;

public class SoftwareProblemReport
{
    protected SoftwareProblemReportPK primaryKey = null;
    protected int scrNo = 0;
    protected java.sql.Timestamp date = new java.sql.Timestamp(0);
    protected String originator = " ";
    protected int state = 1;
    protected String swTitle = " ";
    protected String swVersion = " ";
    protected int priority = 0;
    protected String problemDescription = " ";
    protected String environment = " ";
    protected String recommendedSolution = " ";
    protected String srbDecision = " ";
    protected String senderAddr = " ";
    protected String ccAddr = " ";
    protected java.sql.Timestamp changed = null;
    protected java.sql.Timestamp timestamp = null;
    
    public SoftwareProblemReport()
    {
    }//SoftwareProblemReport
    
    public SoftwareProblemReport(int aProjectId, int aSprNo)
    {
        primaryKey = new SoftwareProblemReportPK(aProjectId, aSprNo);
    }//SoftwareProblemReport
    

    public boolean create(int aProjectId, int aSprNo) throws javax.naming.NamingException, java.sql.SQLException
    {
        primaryKey = new SoftwareProblemReportPK(aProjectId, aSprNo);
        return create();
    }//create
    
    public boolean create() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        Timestamp tm = new Timestamp(System.currentTimeMillis());
        String sqlStr;
        
        if (date==null) date = tm;
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "insert into Software_Problem_Report ("
                   + "project_id, spr_no, scr_no, date, originator, state, sw_title, sw_version, "
                   + "priority, problem_description, environment, recommended_solution, "
                   + "srb_decision, sender_addr, cc_addr, "
                   + "changed, timestamp "
                   + ") values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, primaryKey.projectId);
            ps.setInt(2, primaryKey.sprNo);
            ps.setInt(3, scrNo);
            ps.setTimestamp(4, date);
            ps.setString(5, originator);
            ps.setInt(6, state);
            ps.setString(7, swTitle);
            ps.setString(8, swVersion);
            ps.setInt(9, priority);
            ps.setString(10, problemDescription);
            ps.setString(11, environment);
            ps.setString(12, recommendedSolution);
            ps.setString(13, srbDecision);
            ps.setString(14, senderAddr);
            ps.setString(15, ccAddr);
            ps.setTimestamp(16, tm);
            ps.setTimestamp(17, tm);
            
            if (ps.executeUpdate() != 1) return false;
            
            changed = tm;
            timestamp = tm;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//create

    public boolean remove() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "delete from Software_Problem_Report where "
                   + "project_id=? and spr_no=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, primaryKey.projectId);
            ps.setInt(2, primaryKey.sprNo);
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//remove

    public boolean removeByProjectId(int aProjectId) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "delete from Software_Problem_Report where "
                   + "project_id=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, aProjectId);
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//removeByProjectId

    public boolean store() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        Timestamp tm = new Timestamp(System.currentTimeMillis());
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "update Software_Problem_Report set "
                   + "scr_no=?, date = ?, originator=?, state=?, sw_title=?, sw_version=?, "
                   + "priority=?, problem_description=?, environment=?, "
                   + "recommended_solution=?, srb_decision=?, sender_addr=?, "
                   + "cc_addr=?, changed=? "
                   + "where project_id=? and spr_no=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, scrNo);
            ps.setTimestamp(2, date);
            ps.setString(3, originator);
            byte i = (byte)state;
 System.out.println("State=(char)"+state+"   (byte)"+i);
            ps.setByte(4, i);
//            ps.setInt(4, i);
            ps.setString(5, swTitle);
            ps.setString(6, swVersion);
            ps.setInt(7, priority);
            ps.setString(8, problemDescription);
            ps.setString(9, environment);
            ps.setString(10, recommendedSolution);
            ps.setString(11, srbDecision);
            ps.setString(12, senderAddr);
            ps.setString(13, ccAddr);
            ps.setTimestamp(14, tm);
            ps.setInt(15, primaryKey.projectId);
            ps.setInt(16, primaryKey.sprNo);
            changed = tm;
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//store

    
    // Finder methods
    public SoftwareProblemReportPK findByPrimaryKey(int aProjectId, int aSprNo) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        ResultSet rs;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return null;
            sqlStr = "select * from Software_Problem_Report "
                   + "where project_id=? and spr_no=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return null;
            ps.setInt(1, aProjectId);
            ps.setInt(2, aSprNo);
            
            if ((rs=ps.executeQuery())==null) return null;
            if (!rs.next()) return null;
            primaryKey = new SoftwareProblemReportPK(aProjectId, aSprNo);
            scrNo = rs.getInt("scr_no");
            date = rs.getTimestamp("date");
            originator = rs.getString("originator");
            state = (char)rs.getInt("state");
            swTitle = rs.getString("sw_title");
            swVersion = rs.getString("sw_version");
            priority = rs.getInt("priority");
            problemDescription = rs.getString("problem_description");
            environment = rs.getString("environment");
            recommendedSolution = rs.getString("recommended_solution");
            srbDecision = rs.getString("srb_decision");
            senderAddr = rs.getString("sender_addr");
            ccAddr = rs.getString("cc_addr");
            changed = rs.getTimestamp("changed");
            timestamp = rs.getTimestamp("timestamp");
            
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return primaryKey;
    }//findByPrimaryKey

    public Collection findByProjectId(int aProjectId) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        ResultSet rs;
        SoftwareProblemReport softwareProblemReport = null;
        String sqlStr;
        ArrayList retVal;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return null;
            sqlStr = "select * from Software_Problem_Report "
                   + "where project_id=? order by spr_no desc";
            if ((ps=conn.prepareStatement(sqlStr))==null) return null;
            ps.setInt(1, aProjectId);
            
            if ((rs=ps.executeQuery())==null) return null;
            retVal = new ArrayList();
            while (rs.next()) {
                softwareProblemReport = new SoftwareProblemReport(aProjectId, rs.getInt("spr_no"));
                softwareProblemReport.scrNo = rs.getInt("scr_no");
                softwareProblemReport.date = rs.getTimestamp("date");
                softwareProblemReport.originator = rs.getString("originator");
                softwareProblemReport.state = (char)rs.getInt("state");
                softwareProblemReport.swTitle = rs.getString("sw_title");
                softwareProblemReport.swVersion = rs.getString("sw_version");
                softwareProblemReport.priority = rs.getInt("priority");
                softwareProblemReport.problemDescription = rs.getString("problem_description");
                softwareProblemReport.environment = rs.getString("environment");
                softwareProblemReport.recommendedSolution = rs.getString("recommended_solution");
                softwareProblemReport.srbDecision = rs.getString("srb_decision");
                softwareProblemReport.senderAddr = rs.getString("sender_addr");
                softwareProblemReport.ccAddr = rs.getString("cc_addr");
                softwareProblemReport.changed = rs.getTimestamp("changed");
                softwareProblemReport.timestamp = rs.getTimestamp("timestamp");
                retVal.add(softwareProblemReport);
            }
            
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return retVal;
    }//findByProjectId

    public Collection findByProjectIdScrNo(int aProjectId, int aScrNo) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        ResultSet rs;
        SoftwareProblemReport softwareProblemReport = null;
        String sqlStr;
        ArrayList retVal;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return null;
            sqlStr = "select * from Software_Problem_Report "
                   + "where project_id=? and Scr_No=? order by spr_no";
            if ((ps=conn.prepareStatement(sqlStr))==null) return null;
            ps.setInt(1, aProjectId);
            ps.setInt(2, aScrNo);
            
            if ((rs=ps.executeQuery())==null) return null;
            retVal = new ArrayList();
            while (rs.next()) {
                softwareProblemReport = new SoftwareProblemReport(aProjectId, aScrNo);
                softwareProblemReport.scrNo = rs.getInt("scr_no");
                softwareProblemReport.date = rs.getTimestamp("date");
                softwareProblemReport.originator = rs.getString("originator");
                softwareProblemReport.state = (char)rs.getInt("state");
                softwareProblemReport.swTitle = rs.getString("sw_title");
                softwareProblemReport.swVersion = rs.getString("sw_version");
                softwareProblemReport.priority = rs.getInt("priority");
                softwareProblemReport.problemDescription = rs.getString("problem_description");
                softwareProblemReport.environment = rs.getString("environment");
                softwareProblemReport.recommendedSolution = rs.getString("recommended_solution");
                softwareProblemReport.srbDecision = rs.getString("srb_decision");
                softwareProblemReport.senderAddr = rs.getString("sender_addr");
                softwareProblemReport.ccAddr = rs.getString("cc_addr");
                softwareProblemReport.changed = rs.getTimestamp("changed");
                softwareProblemReport.timestamp = rs.getTimestamp("timestamp");
                retVal.add(softwareProblemReport);
            }
            
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return retVal;
    }//findByProjectIdScrNo

    
    // Public get methods for the primary key
    public SoftwareProblemReportPK getPrimaryKey() {return primaryKey;}
    public int getProjectId() {return primaryKey.projectId;}
    public int getSprNo() {return primaryKey.sprNo;}
    
    // Methods to get/set the rest of the properties
    public int getScrNo() {return scrNo;}
    public void setScrNo(int no) {scrNo = no;}
    
    public java.sql.Timestamp getDate() {return date;}
    public void setDate(java.sql.Timestamp aDate) {date = aDate;}
    
    public String getOriginator() {return originator;}
    public void setOriginator(String aOriginator) {originator = aOriginator;}
    
    public int getState() {return state;}
    public void setState(int aState) {state = aState;}
    
    public String getSwTitle() {return swTitle;}
    public void setSwTitle(String aTitle) {swTitle = aTitle;}
    
    public String getSwVersion() {return swVersion;}
    public void setSwVersion(String aVersion) {swVersion = aVersion;}
    
    public int getPriority() {return priority;}
    public void setPriority(int aPriority) {priority = aPriority;}
    
    public String getProblemDescription() {return problemDescription;}
    public void setProblemDescription(String aDescription) {problemDescription = aDescription;}
    
    public String getEnvironment() {return environment;}
    public void setEnvironment(String aEnvironment) {environment = aEnvironment;}
    
    public String getRecommendedSolution() {return recommendedSolution;}
    public void setRecommendedSolution(String aSolution) {recommendedSolution = aSolution;}
    
    public String getSrbDecision() {return srbDecision;}
    public void setSrbDecision(String aDecision) {srbDecision = aDecision;}
    
    public String getSenderAddr() {return senderAddr;}
    public void setSenderAddr(String aAddr) {senderAddr = aAddr;}
    
    public String getCcAddr() {return ccAddr;}
    public void setCcAddr(String aAddr) {ccAddr = aAddr;}

    public java.sql.Timestamp getChanged() {return changed;}
    public java.sql.Timestamp getTimestamp() {return timestamp;}
    
}//SoftwareProblemReport
