/*
 * Users.java     Auther: Henrik Baastrup (c) 2003
 *
 * This software is licensed under the GNU LGPL (Lesser general Public License):
 *   http://www.gnu.org/copyleft/lesser.html
 *
 *
 * Created on October 1, 2003, 4:40 PM
 */

package com.hbaastrup.AutoReportGenerator.dataLayer;
import java.sql.*;
import java.util.*;

public class Users
{
    public static final int DEFAULT_PROJECT_ID = 1;
    
    protected UsersPK primaryKey = null;
    protected String password = " ";
    protected String eMail = " ";
    protected String initials = " ";
    protected String roles = " ";
    protected String description = " ";
    protected java.sql.Timestamp changed = null;
    protected java.sql.Timestamp timestamp = null;
    
    public Users()
    {
    }//Projects
    
    public Users(UsersPK aPrimaryKey)
    {
        primaryKey = aPrimaryKey;
    }//Projects
    
    public Users(int aProjectId, String aUserId)
    {
        primaryKey = new UsersPK(aProjectId, aUserId);
    }//Projects
    

    public boolean create(int aProjectId, String aUserId) throws javax.naming.NamingException, java.sql.SQLException
    {
        primaryKey = new UsersPK(aProjectId, aUserId);
        return create();
    }//create
    
    public boolean create() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        Timestamp tm = new Timestamp(System.currentTimeMillis());
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "insert into Users ("
                   + "project_id, user_id, password, initials, e_mail, roles, description, changed, timestamp "
                   + ") values (?,?,?,?,?,?,?,?,?)";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, primaryKey.projectId);
            ps.setString(2, primaryKey.userId);
            ps.setString(3, password);
            ps.setString(4, initials);
            ps.setString(5, eMail);
            ps.setString(6, roles);
            ps.setString(7, description);
            ps.setTimestamp(8, tm);
            ps.setTimestamp(9, tm);
            
            if (ps.executeUpdate() != 1) return false;
            
            changed = tm;
            timestamp = tm;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//create

    public boolean remove() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "delete from Users where "
                   + "project_id=? and user_id=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, primaryKey.projectId);
            ps.setString(2, primaryKey.userId);
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//remove

    public boolean removeByProjectId(int aProjectId) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "delete from Users where "
                   + "project_id=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setInt(1, aProjectId);
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//removeByProjectId

    public boolean store() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        Timestamp tm = new Timestamp(System.currentTimeMillis());
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return false;
            sqlStr = "update Users set "
                   + "password=?, initials=?, e_mail=?, roles=?, description=?, changed=? "
                   + "where project_id=? and user_id=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return false;
            ps.setString(1, password);
            ps.setString(2, initials);
            ps.setString(3, eMail);
            ps.setString(4, roles);
            ps.setString(5, description);
            ps.setTimestamp(6, tm);
            ps.setInt(7, primaryKey.projectId);
            ps.setString(8, primaryKey.userId);
            changed = tm;
            
            if (ps.executeUpdate() != 1) return false;
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return true;
    }//store


    // Finder methods
    public UsersPK findByPrimaryKey(int aProjectId, String aUserId) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        ResultSet rs;
        String sqlStr;
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return null;
            sqlStr = "select * from Users "
                   + "where project_id=? and user_id=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return null;
            ps.setInt(1, aProjectId);
            ps.setString(2, aUserId);
            
            if ((rs=ps.executeQuery())==null) return null;
            if (!rs.next()) return null;
            primaryKey = new UsersPK(aProjectId, aUserId);
            password = rs.getString("password");
            initials = rs.getString("initials");
            eMail = rs.getString("e_mail");
            roles = rs.getString("roles");
            description = rs.getString("description");
            changed = rs.getTimestamp("changed");
            timestamp = rs.getTimestamp("timestamp");
            
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return primaryKey;
    }//findByPrimaryKey

    public Collection findAll() throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        ResultSet rs;
        String sqlStr;
        Users usersRec;
        UsersPK usersPkRec;
        ArrayList retVal = new ArrayList();
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return null;
            sqlStr = "select * from Users ";
            if ((ps=conn.prepareStatement(sqlStr))==null) return null;
            
            if ((rs=ps.executeQuery())==null) return null;
            while (rs.next()) {
                usersPkRec = new UsersPK(rs.getInt("project_id"),rs.getString("user_id"));
                usersRec = new Users(usersPkRec);
                usersRec.password = rs.getString("password");
                usersRec.initials = rs.getString("initials");
                usersRec.eMail = rs.getString("e_mail");
                usersRec.roles = rs.getString("roles");
                usersRec.description = rs.getString("description");
                usersRec.changed = rs.getTimestamp("changed");
                usersRec.timestamp = rs.getTimestamp("timestamp");
                retVal.add(usersRec);
            }
            
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return retVal;
    }//findAll

    public Collection findByProjectId(int aProjectId) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        ResultSet rs;
        String sqlStr;
        Users usersRec;
        UsersPK usersPkRec;
        ArrayList retVal = new ArrayList();
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return null;
            sqlStr = "select * from Users where project_id=?";
            if ((ps=conn.prepareStatement(sqlStr))==null) return null;
            ps.setInt(1,aProjectId);
            
            if ((rs=ps.executeQuery())==null) return null;
            while (rs.next()) {
                usersPkRec = new UsersPK(rs.getInt("project_id"),rs.getString("user_id"));
                usersRec = new Users(usersPkRec);
                usersRec.password = rs.getString("password");
                usersRec.initials = rs.getString("initials");
                usersRec.eMail = rs.getString("e_mail");
                usersRec.roles = rs.getString("roles");
                usersRec.description = rs.getString("description");
                usersRec.changed = rs.getTimestamp("changed");
                usersRec.timestamp = rs.getTimestamp("timestamp");
                retVal.add(usersRec);
            }
            
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return retVal;
    }//findByProjectId

    public Collection findByUserIdPassword(String aUserId, String aPassword) throws javax.naming.NamingException, java.sql.SQLException
    {
        Connection conn = null;
        PreparedStatement ps;
        ResultSet rs;
        String sqlStr;
        Users usersRec;
        UsersPK usersPkRec;
        ArrayList retVal = new ArrayList();
        
        try {
            if ((conn=ConnectionHandler.getConnection())==null) return null;
            sqlStr = "select * from Users where user_id=? and password=? ";
            if ((ps=conn.prepareStatement(sqlStr))==null) return null;
            ps.setString(1,aUserId);
            ps.setString(2,aPassword);
            
            if ((rs=ps.executeQuery())==null) return null;
            while (rs.next()) {
                usersPkRec = new UsersPK(rs.getInt("project_id"),rs.getString("user_id"));
                usersRec = new Users(usersPkRec);
                usersRec.password = rs.getString("password");
                usersRec.initials = rs.getString("initials");
                usersRec.eMail = rs.getString("e_mail");
                usersRec.roles = rs.getString("roles");
                usersRec.description = rs.getString("description");
                usersRec.changed = rs.getTimestamp("changed");
                usersRec.timestamp = rs.getTimestamp("timestamp");
                retVal.add(usersRec);
            }
            
        } catch (javax.naming.NamingException ex) {
            throw ex;
        } catch (java.sql.SQLException ex) {
            throw ex;
        } finally {
            try {conn.close();} catch (Exception ignore) {}
        }
        
        return retVal;
    }//findAll

    
    // Public get methods for the primary key
    public UsersPK getPrimaryKey() {return primaryKey;}
    public int getProjectId() {return primaryKey.projectId;}
    public String getUserId() {return primaryKey.userId;}
    
    // Methods to get/set the rest of the properties
    public String getPassword() {return password;}
    public void setPassword(String aPassword) {password = aPassword;}
    
    public String getInitials() {return initials;}
    public void setInitials(String aInitials) {initials = aInitials;}
    
    public String getEMail() {return eMail;}
    public void setEMail(String aAddress) {eMail = aAddress;}
    
//    public String getRoles() {return roles;}
    public ArrayList getRoles()
    {
        if (roles==null) return null;
        ArrayList retVal = new ArrayList();
        String temp = new String(roles);
        int comma;
        while (true) {
             if ((comma = temp.indexOf(',')) < 0) break;
             retVal.add(temp.substring(0,comma).trim());
             temp = temp.substring(comma+1);
        }
        temp = temp.trim();
        if (temp.length() > 0) retVal.add(temp);
        return retVal;
    }//getRoles
    public void setRoles(String roleList) {roles = roleList;}
    public void setRoles(String[] roleList)
    {
        if (roleList==null) {roles=" "; return;}
        StringBuffer rolesBuf = new StringBuffer("");
        for (int i=0; i<roleList.length; i++) {
            rolesBuf.append(roleList[i]);
            if (i < roleList.length-1) rolesBuf.append(',');
        }
        roles = rolesBuf.toString();
    }//setRoles
    public void setRoles(ArrayList roleList)
    {
        if (roleList==null) {roles=" "; return;}
        StringBuffer rolesBuf = new StringBuffer("");
        Iterator iter = roleList.iterator();
        while (iter.hasNext()) {
            if (rolesBuf.length() > 0) rolesBuf.append(',');
            String role = (String)iter.next();
            rolesBuf.append(role);
        }
        roles = rolesBuf.toString();
    }//setRoles
    
    public String getDescription() {return description;}
    public void setDescription(String aDescription) {description = aDescription;}
    
    public java.sql.Timestamp getChanged() {return changed;}
    public java.sql.Timestamp getTimestamp() {return timestamp;}
    
    public boolean doesUserHasRoule(String roule)
    {
        ArrayList roules;
        if ((roules=getRoles())==null) return false;
        Iterator iter = roules.iterator();
        while (iter.hasNext()) {
            String r = (String)iter.next();
            if (r.equals(roule)) return true;
        }
        return false;
    }//doesUserHasRoule
}//Users
