/*
 * JASSRealm.java   Auther: Henrik Baastrup (c) 2003
 *
 * This software is licensed under the GNU LGPL (Lesser general Public License):
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * Created on December 18, 2003, 7:38 PM
 */

package com.hbaastrup.AutoReportGenerator.realm;
import java.util.*;
import java.security.Principal;
import javax.security.auth.Subject;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

public class JAASRealm {
    protected String appName = "AutoReportGenerator";
    protected ArrayList userClasses = new ArrayList();
    protected ArrayList roleClasses = new ArrayList();
    
    
    protected Principal createPrincipal(javax.security.auth.Subject subject)
    {
        String username = null;
        String password = null;
        int projectId = 0;
        ArrayList roles = null;
        
        Iterator iter = subject.getPrincipals().iterator();
        while (iter.hasNext()) {
            Principal princ = (Principal)iter.next();
            String princClass = princ.getClass().getName();
            if (username==null && userClasses.contains(princClass)) username = princ.getName();
            if (roleClasses.contains(princClass) && princ instanceof com.hbaastrup.AutoReportGenerator.realm.GenericPrincipal) {
                String strArr[] = ((GenericPrincipal)princ).getRouls();
                if (strArr!=null) {
                    roles = new ArrayList();
                    for (int i=0; i<strArr.length; i++) roles.add(strArr[i]);
                }
            }
        }
        if (username != null) return new GenericPrincipal(username, password, 0, roles);
        else return null;
    }//createPrincipal
    
    public Principal authenticate (String username, String password, int projectId)
    {
        Subject subject = null;
//        Subject subject = new Subject();
        LoginContext lc = null;
        
        try {
            lc = new LoginContext(appName, new GenericCallBackHandler(username, password, projectId));
//            lc = new LoginContext(appName, subject, new GenericCallBackHandler(username, password, projectId));
        } catch (LoginException ex) {
            ex.printStackTrace();
            return null;
        }
        try {
            lc.login();
            subject = lc.getSubject();
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
        
        Principal principal = createPrincipal(subject);
        return principal;
    }//authenticate
    
    public String getAppName() {return appName;}
    public void setAppName(String aName) {appName = aName;}
    
    // A comma-delimited list of java.security.Principal classes that
    // represent individual users
    protected String userClasssNames = null;
    public void setUserClassNames(String aUserClasssNames)
    {
        userClasssNames = aUserClasssNames;
        userClasses.clear();
        if (aUserClasssNames==null) return;
        String temp = userClasssNames;
        int comma;
        while (true) {
            if ((comma = temp.indexOf(',')) < 0) break;
            userClasses.add(temp.substring(0,comma).trim());
            temp = temp.substring(comma+1);
        }
        temp = temp.trim();
        if (temp.length() > 0) userClasses.add(temp);
    }//setUserClassNames
    
    // A comma-delimited list of java.security.Principal classes that
    // represent security roles
    protected String roleClasssNames = null;
    public void setRoleClassNames(String aRoleClasssNames)
    {
        roleClasssNames = aRoleClasssNames;
        roleClasses.clear();
        if (aRoleClasssNames==null) return;
        String temp = roleClasssNames;
        int comma;
        while (true) {
            if ((comma = temp.indexOf(',')) < 0) break;
            roleClasses.add(temp.substring(0,comma).trim());
            temp = temp.substring(comma+1);
        }
        temp = temp.trim();
        if (temp.length() > 0) roleClasses.add(temp);
    }//setRoleClassNames
        
    
    public static void main(String[] args)
    {
        Properties prop = System.getProperties();
        prop.setProperty("java.security.auth.login.config", "/home/hbaastrup/projects/AutoReportGenerator/WEB-INF/deploy/login.config");
        prop.setProperty("login.configuration.provider", "com.hbaastrup.AutoReportGenerator.realm.GeneralConfiguration");
        prop.list(System.out);
        
        JAASRealm realm = new JAASRealm();
        String genericPrincipalClassName = "com.hbaastrup.AutoReportGenerator.realm.GenericPrincipal";
        realm.setUserClassNames(genericPrincipalClassName);
        realm.setRoleClassNames(genericPrincipalClassName);
        
       
//        Principal princ = realm.authenticate(args[0], args[1]);
        Principal princ = realm.authenticate("admin", "admin",1);
        if (princ==null) System.out.println("NULL Principal");
        else System.out.println("Principal name="+princ.getName());
    }//main
}//JAASRealm
