/*
 * SessionBinder.java
 *
 * This software is licensed under the GNU LGPL (Lesser general Public License):
 *   http://www.gnu.org/copyleft/lesser.html
 */

package com.hbaastrup.AutoReportGenerator.realm;

/**
 *
 * @author  (c) Henrik Baastrup
 */
import java.util.*;
import java.security.Principal;
import javax.security.auth.Subject;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;
import javax.servlet.http.HttpSessionBindingListener;
import com.hbaastrup.AutoReportGenerator.dataLayer.*;

public class SessionBinder implements HttpSessionBindingListener
{
    protected String appName = "AutoReportGenerator";
    private int sessionCounter = 0;
    private LoginContext loginContext = null;
    private Subject sessionSubject = null;
    private GenericPrincipal sessionPrincipal = null;
    private String userInitials = "";
    private String eMail = "";
    private String userDescription = "";
    private String projectDescription = "";
    private String projectSwTitle = "";
    private String projectSwVersion = "";
    
    public SessionBinder()
    {
    }//SessionBinder
    
    public boolean authenticate(String username, String password, int projectId)
    {
System.out.println("SessionBinder.authenticate("+username+","+password+","+projectId+")");
        sessionSubject = null;
        sessionPrincipal = null;
        try {
            loginContext = new LoginContext(appName, new GenericCallBackHandler(username, password, projectId));
        } catch (LoginException ex) {
            ex.printStackTrace();
            return false;
        }
        
        try {
            loginContext.login();
            sessionSubject = loginContext.getSubject();
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
        
        Iterator iter = sessionSubject.getPrincipals().iterator();
        while (iter.hasNext()) {
            Principal princ = (Principal)iter.next();
            if (sessionPrincipal==null && princ instanceof GenericPrincipal) sessionPrincipal = (GenericPrincipal)princ;
        }
        if (sessionPrincipal!=null) {
            try {
                Users users = new Users();
                if (users.findByPrimaryKey(sessionPrincipal.getProjectId(), sessionPrincipal.getName())!=null) {
                    userInitials = users.getInitials();
                    eMail = users.getEMail();
                    userDescription = users.getDescription();
                }
                Projects projects = new Projects();
                if (projects.findByPrimaryKey(sessionPrincipal.getProjectId())!=0) {
                    projectDescription = projects.getDescription();
                    projectSwTitle = projects.getSwTitle();
                    projectSwVersion = projects.getSwVersion();
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
        else {
System.out.println("SessionBinder.authenticate(): sessionPrincipal=null");
        }
        
        return true;
    }//authenticate
    
    public void valueBound(javax.servlet.http.HttpSessionBindingEvent httpSessionBindingEvent)
    {
        sessionCounter++;
System.out.println("SessionBinder.valueBound(): "+sessionCounter);
    }
    
    public void valueUnbound(javax.servlet.http.HttpSessionBindingEvent httpSessionBindingEvent)
    {
        sessionCounter--;
        if (sessionCounter<0) sessionCounter = 0;
System.out.println("SessionBinder.valueUnbound(): "+sessionCounter);
        if (sessionCounter==0 && loginContext!=null) {
            try {
                loginContext.logout();
            } catch (LoginException ex) {
                ex.printStackTrace();
            } finally {
                loginContext = null;
                sessionSubject = null;
                sessionPrincipal = null;
            }
        }
    }
    
    public boolean doesUserHasRoule(String roule)
    {
        if (sessionPrincipal==null) return false;
        String[] roules = sessionPrincipal.getRouls();
        for (int i=0; i< roules.length; i++) {
            if (roules[i].equals(roule)) return true;
        }
        return false;
    }//doesUserHasRoule
    
    public int getSessionCounter() {return sessionCounter;}
    public String getName() {if (sessionPrincipal==null) return null; return sessionPrincipal.getName();}
    public String[] getRouls() {if (sessionPrincipal==null) return null; return sessionPrincipal.getRouls();}
    public int getProjectId() {if (sessionPrincipal==null) return 0; return sessionPrincipal.getProjectId();}
    public String getUserInitials() {return userInitials;}
    public String getEMail() {return eMail;}
    public String getUserDescription() {return userDescription;}
    public String getProjectDescription() {return projectDescription;}
    public String getProjectSwTitle() {return projectSwTitle;}
    public String getProjectSwVersion() {return projectSwVersion;}
    
    
    public static void main(String[] args)
    {
        Properties prop = System.getProperties();
        prop.setProperty("java.security.auth.login.config", "/home/hbaastrup/projects/AutoReportGenerator/WEB-INF/deploy/login.config");
        
        SessionBinder sb = new SessionBinder();
        if (sb.authenticate("admin", "admin", 1)) System.out.println("Login OK!");
        else System.out.println("Login failed");
    }//main
}//SessionBinder
