/*
 * MiniXMLReader.java
 *
 * Created on July 1, 2003, 2:19 PM
 */

package com.hbaastrup.MiniXML;

/**
 *
 * @author  Henrik Baastrup (c) 2003
 * @version 0.1
 *
 * Home page: http://digilander.iol.it/hba/Linux/index.html
 * This software is licensed under the GNU LGPL (Lesser general Public License):
 *   http://www.gnu.org/copyleft/lesser.html
 *
 */
import java.io.*;
import java.util.*;

public class MiniXMLReader 
{
    public static final String VERSION = "0.1";
    public static final int NEW_TAG_STATE = 0; 
    public static final int GET_TAG_NAME_STATE = 1;
    public static final int GET_ATTR_NAME_STATE = 2;
    public static final int GET_ATTR_VALUE_STATE = 3;
    public static final int GET_TAG_VALUE_STATE = 4;
    public static final int NEW_SUB_TAG_STATE = 5;
    public static final int GET_END_TAG_NAME_STATE = 6;
    public static final int TAG_END_STATE = 7;
    public static final int TAG_COMMENT_STATE = 8;
    public static final int TAG_DECLARATION_STATE = 9;


    DefaultXMLReaderHandler xmlHandler = new DefaultXMLReaderHandler();
    Hashtable attributes = new Hashtable();
    
    public MiniXMLReader() {
    }
    
    public void parser(InputStream is) throws MiniXMLException
    {
        StringBuffer tagName = new StringBuffer(256);
        StringBuffer attrName = new StringBuffer(256);
        StringBuffer attrValue = new StringBuffer(256);
        StringBuffer tagValue = new StringBuffer(256);
        StringBuffer endTagName = new StringBuffer(256);
        int state = NEW_TAG_STATE;
        int oldState, pingState;
        int byt, byt1, byt2, byt3;
        boolean ignore, stop, noStep, endStateDone;
        
        xmlHandler.startDocument();
        oldState = 0;
        pingState = 0;
        ignore = false;
        stop = false;
        noStep = false;
        endStateDone = false;
        byt = 0; byt1 = 0; byt2 = 0;
        try {
            while (byt>=0 && !stop) {
                if (noStep) noStep = false;
                else if ((byt=is.read())<0) break;
                
                if (byt=='\r' || byt=='\n') continue; //ignore cariage return values

                switch (state) {
                    case NEW_TAG_STATE: //new tag
                        endStateDone = false;
                        if (ignore) { //If we are in an ignore state change state for the next
                            if (byt=='>') ignore = false;
                        }
                        else if (byt=='<') {
                            if ((byt1=is.read())<0) {stop = true; break;}
                            if (byt1=='/') {
                                endTagName = new StringBuffer(256);
                                state = GET_END_TAG_NAME_STATE;
                            }
                            else if (byt1=='?') {
                                oldState = state;
                                state = TAG_DECLARATION_STATE;
                            }
                            else if (byt1=='!') {
                                if ((byt2=is.read())<0) {stop = true; break;}
                                if (byt2=='-') {
                                    if ((byt3=is.read())<0) {stop = true; break;}
                                    if (byt3=='-') {// This is a comment!
                                        oldState = state;
                                        state = TAG_COMMENT_STATE;
                                    }
                                    else ignore = true;
                                }
                                else ignore = true;
                            }
                            else {
                                tagName = new StringBuffer(256);
                                tagName.append((char)(0xFF&byt1));
                                attributes.clear();
                                state = GET_TAG_NAME_STATE;
                            }
                        }
                        break;

                    case GET_TAG_NAME_STATE://get begin tag name
                        endStateDone = false;
                        if (byt==' ' || byt=='\t') {
                            attrName = new StringBuffer(256);
                            state = GET_ATTR_NAME_STATE;
                            break;
                        }
                        if (byt=='>') {
                            xmlHandler.startElement(tagName.toString(), attributes);
                            tagValue = new StringBuffer(256);
                            state = GET_TAG_VALUE_STATE;
                            break;
                        }
                        if (byt=='/') {
                            if ((byt1=is.read())<0) {stop=true; break;}
                            if (byt1=='>') {
                                xmlHandler.startElement(tagName.toString(), attributes);
                                endTagName = tagName;
                                state = TAG_END_STATE;
                                break;
                            }
                            tagName.append((char)(0xFF&byt));
                            byt = byt1;
                        }
                        tagName.append((char)(0xFF&byt));
                        break;

                    case GET_ATTR_NAME_STATE: //Get attribute name
                        endStateDone = false;
                        if (byt==' ' || byt=='\t') break;
                        if (byt=='=') {
                            attrValue = new StringBuffer(256);
                            state = GET_ATTR_VALUE_STATE;
                            break;
                        }
                        if (byt=='>') {
                            attributes.put(attrName.toString(), attrValue.toString());
                            xmlHandler.startElement(tagName.toString(), attributes);
                            state = GET_TAG_VALUE_STATE;
                            break;
                        }
                        if (byt=='/' ) {
                            if ((byt1=is.read())<0) {stop=true; break;}
                            if (byt1=='>') {
                                attributes.put(attrName.toString(), attrValue.toString());
                                xmlHandler.startElement(tagName.toString(), attributes);
                                endTagName = tagName;
                                state = TAG_END_STATE;
                                break;
                            }
                            attrName.append((char)(0xFF&byt));
                            byt = byt1;
                        }
                        attrName.append((char)(0xFF&byt));
                        break;

                    case GET_ATTR_VALUE_STATE: //Get attribute value
                        endStateDone = false;
                        if (byt=='\'' || byt=='\"') {
                            boolean cont = false;
                            switch (pingState) {
                                case 0:
                                    if (byt=='\'') pingState = 1; // we have a ' and expect the attribute end with a '
                                    else if (byt=='\"') pingState = 2; // we have a " and expect the attribute end with a "
                                    break;
                                case 1:
                                    if (byt=='\'') pingState = 0;
                                    else cont = true;
                                    break;
                                case 2:
                                    if (byt=='\"') pingState = 0;
                                    else cont = true;
                                    break;
                            }
                            if (!cont) break;
                        }
                        if ((byt==' ' || byt=='>' || byt=='/') && pingState==0) { // read the next cahr of the attribute value
                            attributes.put(attrName.toString(), attrValue.toString());
                            if (byt==' ') {
                                attrName = new StringBuffer(256);
                                state = GET_ATTR_NAME_STATE;
                                break;
                            }
                            if (byt=='>') {
                                tagValue = new StringBuffer(256);
                                state = GET_TAG_VALUE_STATE;
                                break;
                            }
                            if (byt=='/') {
                                if ((byt1=is.read())<0) {stop=true; break;}
                                if (byt1=='>') {
                                    xmlHandler.startElement(tagName.toString(), attributes);
                                    endTagName = tagName;
                                    state = TAG_END_STATE;
                                    break;
                                }
                                attrValue.append((char)(0xFF&byt));
                                byt = byt1;
                            }
                        }
                        attrValue.append((char)(0xFF&byt));
                        break;

                    case GET_TAG_VALUE_STATE: //Get tag value
                        endStateDone = false;
                        if (byt=='<') {
                            if ((byt1=is.read())<0) {stop = true; break;}
                            if (byt1=='!') {
                                if ((byt2=is.read())<0) {stop = true; break;}
                                if (byt2=='-') {
                                    if ((byt3=is.read())<0) {stop = true; break;}
                                    if (byt3=='-') {// This is a comment!
                                        oldState = state;
                                        state = TAG_COMMENT_STATE;
                                        break;
                                    }
                                    else {
                                        ignore = true;
                                        state = NEW_TAG_STATE;
                                        break;
                                    }
                                }
                                else {
                                    ignore = true;
                                    state = NEW_TAG_STATE;
                                    break;
                                }
                            }
                            else if (byt1=='/') {
                                endTagName = new StringBuffer(256);
                                int l = tagValue.length();
                                char chars[] = new char[l+1];
                                tagValue.getChars(0, l, chars, 0);
                                chars[l] = '\0';
                                xmlHandler.characters(chars, 0, l);
                                state = GET_END_TAG_NAME_STATE;
                                break;
                            }
                            else {
                                byt2 = byt1;
                                byt1 = byt;
                                int l = tagValue.length();
                                char chars[] = new char[l+1];
                                tagValue.getChars(0, l, chars, 0);
                                chars[l] = '\0';
                                xmlHandler.characters(chars, 0, l);
                                state = NEW_SUB_TAG_STATE;
                                break;
                            }
                        }
                        tagValue.append((char)(0xFF&byt));
                        break;

                    case NEW_SUB_TAG_STATE: //Start to get sub tags from the actual tag
                        endStateDone = false;
                        if (byt1=='<') {
                            if (byt2=='!' && byt=='-') {
                                if ((byt3=is.read())<0) {stop = true; break;}
                                if (byt3=='-') {
                                    oldState = state;
                                    state = TAG_COMMENT_STATE;
                                    break;
                                }
                                else {
                                    ignore = true;
                                    state = NEW_TAG_STATE;
                                }
                            }
                            if (byt2=='/') {
                                endTagName = new StringBuffer(256);
                                state = GET_END_TAG_NAME_STATE;
                                break;
                            }
                            tagName = new StringBuffer(256);
                            tagName.append((char)(0xff&byt2));
                            tagName.append((char)(0xff&byt));
                            attributes.clear();
                            state = GET_TAG_NAME_STATE;
                        }
                        break;

                    case GET_END_TAG_NAME_STATE: //Finish the actual tag
                        if (byt=='>') {
                            state = TAG_END_STATE;
                            noStep = true;
                            break;
                        }
                        endTagName.append((char)(0xff&byt));
                        break;

                    case TAG_END_STATE: //No more values, attrbute or subtag to read. Return!
                        xmlHandler.endElement(endTagName.toString());
                        endStateDone = true;
                        state = NEW_TAG_STATE;
                        break;

                    case TAG_COMMENT_STATE: //We read a comment but we do not save it!
                        if (byt=='-') {
                            if ((byt1=is.read())<0) {stop = true; break;}
                            if (byt1=='-') {
                                if ((byt2=is.read())<0) {stop = true; break;}
                                if (byt2=='>') state = oldState;
                            }
                        }
                        break;

                    case TAG_DECLARATION_STATE: //We read a declaraton but do not save it!
                        if (byt=='?') {
                            if ((byt1=is.read())<0) {stop = true; break;}
                            if (byt1=='>') state = oldState;
                        }
                        break;
                }
            }//while
            
            if (state==TAG_END_STATE && !endStateDone) {
                xmlHandler.endElement(endTagName.toString());
            }
            
            xmlHandler.endDocument();
        } catch (IOException ioex) {
ioex.printStackTrace();
            MiniXMLException ex = new MiniXMLException("IO Exception while reading InputStream", ioex);
            throw ex;
        }
    }//parser
    
    
    public void parse(InputStream is, DefaultXMLReaderHandler drh) throws MiniXMLException
    {
        xmlHandler = drh;
        parser(is);
    }


    public void parse(File f, DefaultXMLReaderHandler drh) throws MiniXMLException
    {
        xmlHandler = drh;
        FileInputStream fis;
        
        try {
            fis = new FileInputStream(f);
        } catch (IOException ioex) {
            MiniXMLException ex = new MiniXMLException("IO Exception while creating a new FileInputStrime from File",ioex);
            throw ex;
        }
        parser(fis);
    }


    public static void main(String[] args)
    {
        FileInputStream f;
        MiniXMLReader xmlReader = new MiniXMLReader();
        TestXMLReaderHandler handler = new TestXMLReaderHandler();
        
        try {
            f = new FileInputStream(args[0]);
        } catch (IOException ex) {
            ex.printStackTrace();
            return;
        }
        
        try {
            xmlReader.parse(f, handler);
        } catch (MiniXMLException ex) {
            ex.printStackTrace();
            return;
        }
        
        try {f.close();} catch (IOException ignore) {}
    }//main
}
