/*
 * ErrorInfoTag.java
 *
 * Created on July 31, 2003, 9:55 Am
 *
 * The <error-info> tag is used to pass error information to a page from a Servlet.
 * The tag look for the string attribute errorInfo, first as an attribute to the tag it self,
 * then in the request class and then in the session class. If the errorInfo attribute is present 
 * (not null) the body part of the tag will be skip and the content of the attribute will be written
 * instead.
 * The tag take the following attributes:
 * fontClass='style sheet class': Will insert the text in the errorInfo attribute in the body of a
 *           font tag with the class attribute set the value of fontClass.
 *
 * <error-info [fontClass='style sheet class']>
 * </error-info>
 *
 * Example:
 * A servlet or JSP has set the attribute errorInfo to "ERROR wrong date"
 * Our JSP:
 *   <error-info fontClass='BigRedBold'>
 *   <table>
 *     <tr><td>The date is:</td><td><input type=text></td></tr>
 *   </table>
 *   </error-info>
 * Output:
 * <font class='BigRedBold'>ERROR wrong date</font>
 *
 */

package com.hbaastrup.tagLib;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import java.util.*;
import java.io.*;

public class ErrorInfoTag extends BodyTagSupport
{
    private String errorInfo = null;
    private String fontClass = null;
    
    public ErrorInfoTag()
    {
        super();
    }//ErrorInfoTag
    
    
    public int doStartTag() throws JspException
    {
        JspWriter out;
        
        if (errorInfo==null || "".equals(errorInfo)) {
            javax.servlet.ServletRequest request = pageContext.getRequest();
            if ((errorInfo = (String)request.getAttribute("errorInfo"))==null) {
                javax.servlet.http.HttpSession session = pageContext.getSession();
                errorInfo = (String)session.getAttribute("errorInfo");
            }
        }
        if (errorInfo==null) return EVAL_BODY_INCLUDE;
        
        try {
            out = pageContext.getOut();
            if (fontClass!=null) {
                out.println("<font class='"+fontClass+"'>"+errorInfo+"</font>");
            }
            else out.println(errorInfo);
        } catch (IOException ex) {
            throw new JspException("JsArrayTag: "+ex.toString());
        }
        
        errorInfo = null;
        fontClass = null;
        
        return SKIP_BODY;
    }//doStartTag

    public int doEndTag()
    {
        return EVAL_PAGE;
    }//doEndTag

    
    public void setErrorInfo(String infoStr) {errorInfo = infoStr;}
    public void seterrorinfo(String infoStr) {errorInfo = infoStr;}
    
    public void setFontClass(String aClass) {fontClass = aClass;}
    public void setFontclass(String aClass) {fontClass = aClass;}
}//ErrorInfoTag
