/*
 * TableSelectTag.java
 *
 * Created on February 20, 2003, 9:43 AM
 *
The <table-select> tag create a HTML select list based on values from a request to the database
trough a data layer class. The tag does not take (ignore) a body part, but take the following
attributes:
table='table name': Required. The tag try to load the class in the data layer there represent
      the actual table. Ex: 'My_Table' will match the class com.eds.arc3.dataLayer.MyTable.
option='table field name' Required. This is the table field, witch values, will occur between
       the <option> tags in the <select> list. The above class must have a get method for this
       field. Ex: 'My_Field' match the method getMyField.
name='CGI name': Optional. This is the name of the <select> list in the HTML page.
onChange='javaScript function': Optional. A JavaScript function there will be called when the list
         change value.
value='table field name': Optional. This is the table field which values will occur in the value
      attribute of the option tag. The above class must have a get method for this field.
      Ex: 'My_Field' match the method getMyField.
findBy='data layer finder method': Optional. This is the data layer class's finder method, used
       to create the list. The tag look for a finBy[value of attribute] method in the above class.
       Ex: 'My_name' match the method findByMyName. If this attribute is not existent the tag will
       try to use the findAll method.
findByParam='parameter to the above finder method': Optional. A parameter to the above method.
selected='a value in the list': Optional. This attribute can also be passed by either the request
         class or the session class. The value can either be a value of a option tag, if the option
         tags does not contain value attributes, or a value attribute value in the option tag.
nullOption='text': Optional. Will show the text in the list with the value 0.

<table-select
 table='table name'
 option='table field name'
 [name='CGI name']
 [onChange='JavaScript function']
 [value='table field name']
 [findBy='data layer finder method']
 [findByParam='parameter to the above finder method']
 [selected='value there are selected in the list']
/>
 *
 */

package com.hbaastrup.tagLib;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import java.util.*;
import java.io.*;
import java.lang.reflect.*;
/**
 *
 * @author  Henrik Baastrup (c) 2003
 */
public class TableSelectTag extends TagSupport
{
    private String name = null;
    private String table = null;
    private String onChange = null;
    private String option = null;
    private String optionValue = null;
    private String findBy = null;
    private String findByParam = null;
    private String selectedValue = null;
    private String selectedAttrName = null;
    private String nullOption = null;
    
    public TableSelectTag()
    {
    }//TableSelectTag
    
    private String converteName(String name)
    {
        byte bytArr[];
        int i, j;

        bytArr = name.toLowerCase().getBytes();
        j = 0;
        if (bytArr[0]>= 'a' && bytArr[0] <='z') bytArr[0] += 'A' - 'a';
        for (i=0; i< bytArr.length; i++) {
            if (bytArr[i]=='_') {
                for (j=i+1; j<bytArr.length; j++) bytArr[j-1] = bytArr[j];
                bytArr[bytArr.length-1] = ' ';
                if (bytArr[i]>= 'a' && bytArr[i] <='z') bytArr[i] += 'A' - 'a';
            }
        }
        return new String(bytArr).trim();
    }//converteName
    
    public int doStartTag() throws JspException
    {
        javax.servlet.ServletRequest request;
        Collection coll;
        JspWriter out;
        String finderMethod;
        Class findParams[] = null;
        Object passParam[] = null;
        int optInx, optValInx, findByInx;
        String selectedVal;
        int i, j;
        
        if (table==null) return SKIP_BODY;
        if (option==null) return SKIP_BODY;
        
        request = pageContext.getRequest();
        if (selectedAttrName!=null) {
            if ((selectedVal=(String)request.getAttribute(selectedAttrName))==null) {
                javax.servlet.http.HttpSession session = pageContext.getSession();
                selectedVal = (String)session.getAttribute(selectedAttrName);
            }
        }
        else if (selectedValue==null) {
            if ((selectedVal=(String)request.getAttribute("selected"))==null) {
                javax.servlet.http.HttpSession session = pageContext.getSession();
                selectedVal = (String)session.getAttribute("selected");
            }
        }
        else selectedVal = selectedValue;

        if (findByParam==null) {
            if ((findByParam=(String)request.getAttribute("findByParam"))==null) {
                javax.servlet.http.HttpSession session = pageContext.getSession();
                findByParam = (String)session.getAttribute("findByParam");
            }
        }

        //format table name to fit at the names in our data layer
        table = converteName(table);
        option = converteName(option);
        if (optionValue!=null) optionValue = converteName(optionValue);
        if (findBy!=null) finderMethod = "findBy"+converteName(findBy);
        else finderMethod = "findAll";
        
        try {
            String className = "com.hbaastrup.AutoReportGenerator.dataLayer."+table;
            Class classDeff = Class.forName(className);
            Object objDeff = classDeff.newInstance();
            Method[] dlMethods = classDeff.getMethods();
            
            optInx = -1;
            optValInx = -1;
            findByInx = -1;
            for (i=0; i<dlMethods.length; i++) {
                if (dlMethods[i].getName().equals("get"+option)) optInx = i;
                if (dlMethods[i].getName().equals("get"+optionValue)) optValInx = i;
                if (dlMethods[i].getName().equals(finderMethod)) findByInx = i;
            }
            
            out = pageContext.getOut();
            if (optInx==-1 || findByInx==-1) {
                out.println("<select></select>");
                return SKIP_BODY;
            }
            
            findParams = dlMethods[findByInx].getParameterTypes();
            if (findByParam!=null && findParams.length>0) {
                passParam = new Object[1];
                if (findParams[0].isPrimitive()) {
                    String typName = findParams[0].getName();
                    if (typName.equals("int")) {
                        passParam[0] = new Integer(findByParam);
                    }
                }
                else if (findParams[0].isInstance(new String())) passParam[0] = findByParam.getClass();
            }
            if ((coll=(Collection)dlMethods[findByInx].invoke(objDeff, passParam))==null) {
                if (nullOption==null) out.println("<select></select>");
                else  out.println("<select><option value='0'>"+nullOption+"</option></select>");
                return SKIP_BODY;
            }
            
            out.print("<select");
            if (name!=null) out.print(" name='"+name+"'");
            if (onChange!=null) out.print(" onChange='"+onChange+"'");
            out.println(">");
            if (nullOption!=null) out.println("<option value='0'>"+nullOption+"</option>");

            Iterator iter = coll.iterator();
            String valStr = null;
            String optStr = null;
            while (iter.hasNext()) {
                Object obj = iter.next();
                optStr = ""+dlMethods[optInx].invoke(obj, null);
                out.print("<option");
                if (optValInx > -1) {
                    valStr = ""+dlMethods[optValInx].invoke(obj, null);
                    if (selectedVal!=null) {if (valStr.equals(selectedVal)) out.print(" selected");}
                    out.print(" value='"+valStr+"'>");
                }
                else {
                    if (selectedVal!=null) {if (optStr.equals(selectedVal)) out.print(" selected");}
                    out.print(">");
                }
 
                if(optStr.length()>23)
                    optStr = optStr.substring(0, 22) + "...";
                
                out.print(optStr);
                out.println("</option>");
            }
            out.println("</select>");
        } catch (Exception ex) {
            throw new JspException("TableSelectTag: "+ex.toString());
        }
        
        return SKIP_BODY;
    }//doStartTag
    
    public int doEndTag()
    {
        return EVAL_PAGE;
    }//doAfterBody

    public void setName(String aName){name = aName;}
    public void setTable(String aTable) {table = aTable;}
    public void setOnChange(String aString) {onChange = aString;}
    public void setOnchange(String aString) {onChange = aString;}
    public void setOption(String aFieldName) {option = aFieldName;}
    public void setValue(String aFieldName) {optionValue = aFieldName;}
    public void setFindBy(String aFieldName) {findBy = aFieldName;}
    public void setFindby(String aFieldName) {findBy = aFieldName;}
    public void setFindByParam(String aValue) {findByParam = aValue;}
    public void setFindbyparam(String aValue) {findByParam = aValue;}
    public void setSelected(String aValue) {selectedValue = aValue;}
    public void setSelectedAttrName(String aAttrName) {selectedAttrName = aAttrName;}
    public void setSelectedattrname(String aAttrName) {selectedAttrName = aAttrName;}
    public void setNullOption(String aText) {nullOption = aText;}
    public void setNulloption(String aText) {nullOption = aText;}


    public static void main(String[] args)
    {
        TableSelectTag tag = new TableSelectTag();
/*        tag.setTable("Data_Collector_Params");
        tag.setOption("Param_Id");
        tag.setFindBy("Canal_Id");
        tag.setFindByParam("1");*/
/*        tag.setTable("Errore_Atm");
        tag.setOption("Errore_Id");*/
        tag.setTable("Data_Collector_Canals");
        tag.setOption("Descrizione");
        tag.setValue("Canal_Id");
        try {
            tag.doStartTag();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}//TableSelectTag
