/*
 * tabFillTag.java
 *
 * Created on February 12, 2003, 9:09 AM
 *
The <table-td> tag creates a content of a table, the <tr> <td> tags. The tag expect that
either the attribute tableTdValues exist or that the valuePasserAttr is set. These attributes
are either in the request class or in the session class. The tag control first the request
class, then, if the attribute is not found, the session class. The attribute must contain a
Vector class, containing a vector of String classes.
The tag take the following attributes:
valuePasserAttr='attribute' Optional. This attribute give the possibility to pass values other
                            than the default attribute (tableTdValues). This attribute is used
                            when there exist more than one table-td tag on a page.
trClass1='style sheet class' Optional. These attributes are style sheet class names and will be,
trClass2='style sheet class'           if exist, set in the <tr> tags.

The tag interpret the body and will substitute dollar sign ($) with the values in the
tableTdValues attribute and number signs (#) with the actual line number. The body of the tag
ill be repeated the necessary times depended on how many values there are in the tableTdTag
attribute.

<table-td [valuePasserAttr='attribute'] [trClass1='class' | trClass2='class']>
{$|#}
</table-td>

Example:
A servlet or a JSP has set the attribute tableTdValues in either the request class or in the
session class with the values {"1", "2", "3", "4", "5", "6", "7"}
Our JSP:
<table border='1'>
<arc3tag:table-td tdClass1='ssClass1' tdClass2='ssClass2'>
<tr><td>$</td><td>$</td><td>This is line number: #</td></tr>
</arc3tag:table-td>
</table>
Output:
<table border='1'>
<tr class='ssClass1'><td>1</td><td>2</td><td>This is line number: 0</td></tr>
<tr class='ssClass2'><td>3</td><td>4</td><td>This is line number: 1</td></tr>
<tr class='ssClass1'><td>5</td><td>6</td><td>This is line number: 2</td></tr>
<tr class='ssClass2'><td>7</td><td>&nbsp;</td><td>This is line number: 3</td></tr>
</table>
 *
 */

package com.hbaastrup.tagLib;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import java.util.*;
import java.io.*;

/**
 *
 * @author  Henrik Baastrup (c) 2003
 */
public class TableTdTag extends BodyTagSupport
{
    private String valuePasserAttr = "tableTdValues";
    private String trClass1 = null;
    private String trClass2 = null;
    private Vector tableValues = null;
    
    
    public TableTdTag()
    {
        super();
    }//TableTdTag
    
    
    public int doStartTag() throws JspException
    {
        return EVAL_BODY_BUFFERED;
    }//doStartTag
    
    public int doEndTag()
    {
        return EVAL_PAGE;
    }//doAfterBody
    
    public int doAfterBody() throws JspException
    {
        BodyContent bodyContent;
        String body;
        String format;
        StringBuffer strBuff;
        Vector valueVector = new Vector(0);
        Vector numVect = new Vector(0);
        boolean usedVal, usedNum;
        JspWriter out;
        Integer inx1, inx2, numInx, valInx;
        int i, j, trInxEnd;
        
        if ((bodyContent=getBodyContent())==null) return SKIP_BODY;
        body = bodyContent.getString();
        
        if (body==null) format = "<tr></tr>";
        else format = new String(body);
        
        if ((i=format.indexOf("<tr")) < 0) {
            format = "<tr>"+format+"</tr>";
            i = 0;
        }
        trInxEnd = format.indexOf('>',i);

        j = 0;
        do {
            if ((i=format.indexOf('$',j+1)) >= 0) valueVector.addElement(new Integer(i));
            j = i;
        } while (j > 0);
        do {
            if ((i=format.indexOf('#',j+1)) >= 0) numVect.addElement(new Integer(i));
            j = i;
        } while (j > 0);
        
        // Get the actual values
        javax.servlet.ServletRequest request = pageContext.getRequest();
        if ((tableValues = (Vector)request.getAttribute(valuePasserAttr))==null) {
            javax.servlet.http.HttpSession session = pageContext.getSession();
            tableValues = (Vector)session.getAttribute(valuePasserAttr);
        }
        try {
            out = bodyContent.getEnclosingWriter();
            
            if (tableValues==null) {
                out.println("<tr><td>&nbsp;</td></tr>");
                return SKIP_BODY;
            }
            
            //do the output
            int line = 0;
            Enumeration tabEnum = tableValues.elements();
            while (tabEnum.hasMoreElements()) {
                strBuff = new StringBuffer(format.substring(0,trInxEnd));
                if (trClass1 != null) {
                    strBuff.append(" class='");
                    if (trClass2!=null && line%2==1) strBuff.append(trClass2+"'>");
                    else strBuff.append(trClass1+"'>");
                    
                }
                else strBuff.append(">");
                
                Enumeration valEnum = valueVector.elements();
                Enumeration numEnum = numVect.elements();
                inx1 = new Integer(trInxEnd);
                valInx = inx1;
                numInx = inx1;
                usedVal = true;
                usedNum = true;
                while (usedVal || usedNum) {
                    if (usedVal) {
                        if (valEnum.hasMoreElements()) {
                            valInx = (Integer)valEnum.nextElement();
                        }
                        else valInx = new Integer(-1);
                        usedVal = false;
                    }
                    if (usedNum) {
                        if (numEnum.hasMoreElements()) {
                            numInx = (Integer)numEnum.nextElement();
                        }
                        else numInx = new Integer(-1);
                        usedNum = false;
                    }
                    
                    if (valInx.intValue() < 0 && numInx.intValue() < 0) break;
                    else if (valInx.intValue() < 0) {inx2 = numInx; usedNum = true;}
                    else if (numInx.intValue() < 0) {inx2 = valInx; usedVal = true;}
                    else if(valInx.intValue() <= numInx.intValue()) {inx2 = valInx; usedVal = true;}
                    else if(valInx.intValue() > numInx.intValue()){inx2 = numInx; usedNum = true;}
                    else inx2 = new Integer(format.length()-1);
    
                    strBuff.append(format.substring(inx1.intValue()+1, inx2.intValue()));
                    
                    if (usedVal) {
                        if (tabEnum.hasMoreElements()) {
                            String tabVal = (String)tabEnum.nextElement();
                            strBuff.append(tabVal);
                        }
                        else strBuff.append("&nbsp;");
                    }
                    else if (usedNum) {
                        strBuff.append(""+line);
                    }
                    inx1 = inx2;
                }
                line++;
                strBuff.append(format.substring(inx1.intValue()+1));
                out.println(strBuff.toString());
            }
        } catch (Exception ex) {
            throw new JspException("TableTdTag: "+ex.toString());
        }
        return SKIP_BODY;
    }
    

    public void setValuePasserAttr(String aClass) {valuePasserAttr = aClass;}
    public void setValuepasserattr(String aClass) {valuePasserAttr = aClass;}
    public void setTrClass(String aClass) {trClass1 = aClass;}
    public void setTrclass(String aClass) {trClass1 = aClass;}
    public void setTrClass1(String aClass) {trClass1 = aClass;}
    public void setTrclass1(String aClass) {trClass1 = aClass;}
    public void setTrClass2(String aClass) {trClass2 = aClass;}
    public void setTrclass2(String aClass) {trClass2 = aClass;}
    public Vector geTableValues() {return tableValues;}
    
}//TableTdTag
