/*
 * Filename Base64EncodeDataSource.java   Auther: Henrik Baastrup (c) 2001
 *
 * This software is licensed under the GNU LGPL (Lesser general Public License):
 *   http://www.gnu.org/copyleft/lesser.html
 *
 */

package com.hbaastrup.util;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import javax.activation.DataSource;

public class Base64EncodeDataSource implements javax.activation.DataSource
{
    static byte tbl[] = {
        'A','B','C','D','E','F','G','H',
        'I','J','K','L','M','N','O','P',
        'Q','R','S','T','U','V','W','X',
        'Y','Z','a','b','c','d','e','f',
        'g','h','i','j','k','l','m','n',
        'o','p','q','r','s','t','u','v',
        'w','x','y','z','0','1','2','3',
        '4','5','6','7','8','9','+','/'
    };
    byte[] bytes;
    int length;
    String contentType;
    String name;
    boolean selected = true;

    public Base64EncodeDataSource(byte[] bytes, String contentType, String name)
    {
        byte src[];
        int i, j;

        if (null == contentType) this.contentType = "application/base64";
        else this.contentType = contentType;
        this.name = name;

        if (bytes!=null) length = 4*((bytes.length+2)/3);
        else length = 0;
        if (length==0) {
            length = 4;
            this.bytes = new byte[4];
            this.bytes[0] = this.bytes[1] = 'A';
            this.bytes[2] = this.bytes[3] = '=';
            return;
        }
        j = 0;
        this.bytes = new byte[length];
        // Transform the 3x8 bits to 4x6 bits, as required by base64.
        if (bytes.length==1) {
            this.bytes[j] = tbl[(bytes[0] >> 2) & 0x3f]; j++;
            this.bytes[j] = tbl[((bytes[0] & 3) << 4)]; j++;
        }
        else if (bytes.length==2) {
            this.bytes[j] = tbl[bytes[0] >> 2]; j++;
            this.bytes[j] = tbl[((bytes[0] & 3) << 4) + ((bytes[1] >> 4) & 0xf)]; j++;
            this.bytes[j] = tbl[((bytes[1] & 0xf) << 2)]; j++;
        }
        else {
            for (i=0; i<bytes.length-2; i+=3) {
                this.bytes[j] = tbl[(bytes[i] >> 2) & 0x3f]; j++;
                this.bytes[j] = tbl[((bytes[i] & 3) << 4) + ((bytes[i+1] >> 4) & 0xf)]; j++;
                this.bytes[j] = tbl[((bytes[i+1] & 0xf) << 2) + ((bytes[i+2] >> 6) & 0x3)]; j++;
                this.bytes[j] = tbl[bytes[i+2] & 0x3f]; j++;
            }
            if (i==bytes.length-1) {
                this.bytes[j] = tbl[(bytes[i] >> 2) & 0x3f]; j++;
                this.bytes[j] = tbl[((bytes[i] & 3) << 4)]; j++;
            }
            else if (i==bytes.length-2) {
                this.bytes[j] = tbl[(bytes[i] >> 2) & 0x3f]; j++;
                this.bytes[j] = tbl[((bytes[i] & 3) << 4) + ((bytes[i+1] >> 4) & 0xf)]; j++;
                this.bytes[j] = tbl[((bytes[i+1] & 0xf) << 2)]; j++;
            }
        }

        // Pad the result if necessary... 
        if (j==length-1) this.bytes[j] = '=';
        else if (j==length-2) this.bytes[j] = this.bytes[j+1] = '=';
    }

    public String getContentType() {return contentType;}

    public String getName() {return name;}

    public int getLength() {return length;}

    public InputStream getInputStream()
    {
        return new ByteArrayInputStream(bytes, 0, length);
    }//getInputStream

    public OutputStream getOutputStream() throws java.io.IOException
    {
        throw new FileNotFoundException();
    }//getOutputStream

    public void setSelected(boolean b) {selected = b;}
    public boolean getSelected() {return selected;}
    public boolean isSelected() {return selected;}
    public byte[] getBytes() {return bytes;}
}//Base64EncodeDataSource

