/*
 * SortablyVector.java
 *
 * Created on November 25, 2003
 */

package com.hbaastrup.util;
import java.util.*;
import java.lang.*;
import java.lang.reflect.*;

/**
 *
 * @author  Henrik Baastrup (c) 2003
 * @version 0.1
 *
 * Home page: http://digilander.iol.it/hba/Linux/index.html
 * This software is licensed under the GNU LGPL (Lesser general Public License):
 *   http://www.gnu.org/copyleft/lesser.html
 *
 */
public class SortablyVector extends Vector
{
    /*
     * This class extend the Vector class so you can do sorting on a Vector
     * there contains Java Beans. The sorting methods use a attribute name to
     * sort after, so all the Bean classes in the Vector has to implement
     * the get method for this attribute.
     */
    public SortablyVector()
    {
        super();
    }//SortablyVector

    public SortablyVector(Collection c)
    {
        super(c);
    }//SortablyVector

    public SortablyVector(int initialCapacity)
    {
        super(initialCapacity);
    }//SortablyVector

    public SortablyVector(int initialCapacity, int capacityIncrement)
    {
        super(initialCapacity, capacityIncrement);
    }//SortablyVector

    
    private String getMethodName(String attrName)
    {
        byte[] chars = attrName.getBytes();
        
        if (chars[0]<'A' || chars[0]>'Z') chars[0] += 'A'-'a';
        return "get"+new String(chars);
    }//getMethodName
    
    
    private int compare(Object o1, Object o2, String attrName) throws NoSuchMethodException
    // Returns:
    //  a negative integer, zero, or a positive integer as the first argument
    //  is less than, equal to, or greater than the second.
    {
        Class cls = o1.getClass();
        Method getMethod = null;
        Object retO1 = null, retO2 = null;
        String metName = getMethodName(attrName);
        
        try {
            getMethod = cls.getMethod(metName,  null);
            retO1 = getMethod.invoke(o1, null);
            cls = o2.getClass();
            getMethod = cls.getMethod(metName,  null);
            retO2 = getMethod.invoke(o2,  null);
        } catch (NoSuchMethodException ex) {
            throw ex;
        } catch (Exception ex) {
            NoSuchMethodException throwEx = new NoSuchMethodException(ex.toString());
            throw throwEx;
        }
        
        
        if (retO1.getClass().isPrimitive()) {
            if (retO1.getClass().getName().equals("int")) {
                int i1 = ((Integer)retO1).intValue();
                int i2 = ((Integer)retO2).intValue();
                
                if (i1 < i2) return -1;
                else if (i1 == i2) return 0;
                else return 1;
            }
            else if (retO1.getClass().getName().equals("long")) {
                long l1 = ((Long)retO1).longValue();
                long l2 = ((Long)retO2).longValue();
                
                if (l1 < l2) return -1;
                else if (l1 == l2) return 0;
                else return 1;
            }
        }
        else if (retO1.getClass().isInstance(new Integer(0))) return ((Integer)retO1).compareTo((Integer)retO2);
        else if (retO1.getClass().isInstance(new Long(0))) return ((Long)retO1).compareTo((Long)retO2);
        else if (retO1.getClass().isInstance(new String())) return ((String)retO1).compareTo((String)retO2);
        else if (retO1.getClass().isInstance(new Date())) return ((Date)retO1).compareTo((Date)retO2);
        else if (retO1.getClass().isInstance(new java.sql.Date(0))) return ((Date)retO1).compareTo((Date)retO2);
        else if (retO1.getClass().isInstance(new java.sql.Timestamp(0))) return ((Date)retO1).compareTo((Date)retO2);
        
        NoSuchMethodException throwEx = new NoSuchMethodException("Can not compare objects");
        throw throwEx;
    }//compare
    
    
    private void swapElms(int i, int j)
    {
        Object obj = this.elementAt(i);
        this.setElementAt(this.elementAt(j), i);
        this.setElementAt(obj, j);
    }//swapElms
    
    
    public SortablyVector hsort(String attrName, int fromElm, int toElm, boolean inc) throws NoSuchMethodException
    //Heap sort algorithm
    {
        if (this.size()==0) return this;
        int n = toElm - fromElm;
        if (n < 0) return this;
        if (n+fromElm > this.size()) return this;
        
        for (int gap=n/2; 0<gap; gap/=2) {
            for (int i=gap+fromElm; i<n+fromElm; i++) {
                for (int j=i-gap; fromElm<=j; j-=gap) {
                    if (inc) { //Increasing
                        if (compare(this.elementAt(j), this.elementAt(j+gap), attrName) > 0) swapElms(j, j+gap);
                    }
                    else { //Decreasing
                        if (compare(this.elementAt(j), this.elementAt(j+gap), attrName) < 0) swapElms(j, j+gap);
                    }
                }
            }
        }
        return this;
    }//hsort

    
    public SortablyVector sortBy(String attrName, int fromElm, int toElm, boolean inc) throws NoSuchMethodException
    // Sort the vector of this class from the index pointed by the fromElm to,
    // bot not with, the toElm. The elements are sorted into increasing order if
    // the inc is true, els in decreasing order.
    {
        return hsort(attrName, fromElm, toElm, inc);
//        return qsort(attrName, fromElm, toElm-1, inc);
    }//sortBy

    public SortablyVector sortBy(String attrName, boolean inc) throws NoSuchMethodException
    {
        return sortBy(attrName, 0, this.size(), inc);
    }//sortBy

    
    public SortablyVector sortBy(String attrName1, boolean inc1, String attrName2, boolean inc2) throws NoSuchMethodException
    {
        int startElm;
        
        if (this.size()==0) return this;
        
        sortBy(attrName1, 0, this.size(), inc1);
        
        startElm = 0;
        for (int i=1; i<this.size(); i++) {
            if (compare(this.elementAt(i-1), this.elementAt(i), attrName1)!=0) {
                sortBy(attrName2, startElm, i-1, inc2);
                startElm = i;
            }
        }
        sortBy(attrName2, startElm, this.size(), inc2);
        return this;
    }//sortBy
    
    
    public SortablyVector sortBy(String attrName1, boolean inc1, String attrName2, boolean inc2, String attrName3, boolean inc3) throws NoSuchMethodException
    {
        int startElm;
        
        if (this.size()==0) return this;
        
        sortBy(attrName1, 0, this.size(), inc1);
        
        startElm = 0;
        for (int i=1; i<this.size(); i++) {
            if (compare(this.elementAt(i-1), this.elementAt(i), attrName1)!=0) {
                sortBy(attrName2, startElm, i-1, inc2);
                startElm = i;
            }
        }
        sortBy(attrName2, startElm, this.size(), inc2);

        startElm = 0;
        for (int i=1; i<this.size(); i++) {
            if (compare(this.elementAt(i-1), this.elementAt(i), attrName2)!=0) {
                sortBy(attrName3, startElm, i-1, inc3);
                startElm = i;
            }
        }
        sortBy(attrName3, startElm, this.size(), inc3);

        return this;
    }//sortBy
    
    public SortablyVector sortBy(String attrName1, boolean inc1, String attrName2, boolean inc2, String attrName3, boolean inc3, String attrName4, boolean inc4) throws NoSuchMethodException
    {
        int startElm;
        
        if (this.size()==0) return this;
        
        sortBy(attrName1, 0, this.size(), inc1);
        
        startElm = 0;
        for (int i=1; i<this.size(); i++) {
            if (compare(this.elementAt(i-1), this.elementAt(i), attrName1)!=0) {
                sortBy(attrName2, startElm, i-1, inc2);
                startElm = i;
            }
        }
        sortBy(attrName2, startElm, this.size(), inc2);

        startElm = 0;
        for (int i=1; i<this.size(); i++) {
            if (compare(this.elementAt(i-1), this.elementAt(i), attrName2)!=0) {
                sortBy(attrName3, startElm, i-1, inc3);
                startElm = i;
            }
        }
        sortBy(attrName3, startElm, this.size(), inc3);

        startElm = 0;
        for (int i=1; i<this.size(); i++) {
            if (compare(this.elementAt(i-1), this.elementAt(i), attrName3)!=0) {
                sortBy(attrName4, startElm, i-1, inc4);
                startElm = i;
            }
        }
        sortBy(attrName4, startElm, this.size(), inc4);

        return this;
    }//sortBy

    public SortablyVector sortBy(String attrName1, boolean inc1, String attrName2, boolean inc2, String attrName3, boolean inc3, String attrName4, boolean inc4, String attrName5, boolean inc5) throws NoSuchMethodException
    {
        int startElm;
        
        if (this.size()==0) return this;
        
        sortBy(attrName1, 0, this.size(), inc1);
        
        startElm = 0;
        for (int i=1; i<this.size(); i++) {
            if (compare(this.elementAt(i-1), this.elementAt(i), attrName1)!=0) {
                sortBy(attrName2, startElm, i-1, inc2);
                startElm = i;
            }
        }
        sortBy(attrName2, startElm, this.size(), inc2);

        startElm = 0;
        for (int i=1; i<this.size(); i++) {
            if (compare(this.elementAt(i-1), this.elementAt(i), attrName2)!=0) {
                sortBy(attrName3, startElm, i-1, inc3);
                startElm = i;
            }
        }
        sortBy(attrName3, startElm, this.size(), inc3);

        startElm = 0;
        for (int i=1; i<this.size(); i++) {
            if (compare(this.elementAt(i-1), this.elementAt(i), attrName3)!=0) {
                sortBy(attrName4, startElm, i-1, inc4);
                startElm = i;
            }
        }
        sortBy(attrName4, startElm, this.size(), inc4);

        startElm = 0;
        for (int i=1; i<this.size(); i++) {
            if (compare(this.elementAt(i-1), this.elementAt(i), attrName4)!=0) {
                sortBy(attrName5, startElm, i-1, inc5);
                startElm = i;
            }
        }
        sortBy(attrName5, startElm, this.size(), inc5);

        return this;
    }//sortBy

    
    
    public Object contain(Object obj, String attrName1) throws NoSuchMethodException
    // If the vector contain an object with the same attribute value as the requested
    // this method will retuirn the object else null
    {
        Iterator iter = this.iterator();
        while (iter.hasNext()) {
            Object inObj = (Object)iter.next();
            if (compare(obj, inObj, attrName1)==0) return inObj;
        }
        return null;
    }//contain
    
    public Object contain(Object obj, String attrName1, String attrName2) throws NoSuchMethodException
    {
        Iterator iter = this.iterator();
        while (iter.hasNext()) {
            Object inObj = (Object)iter.next();
            if (compare(obj, inObj, attrName1)==0 && 
                compare(obj, inObj, attrName2)==0) return inObj;
        }
        return null;
    }//contain
    
    public Object contain(Object obj, String attrName1, String attrName2, String attrName3) throws NoSuchMethodException
    {
        Iterator iter = this.iterator();
        while (iter.hasNext()) {
            Object inObj = (Object)iter.next();
            if (compare(obj, inObj, attrName1)==0 && 
                compare(obj, inObj, attrName2)==0 &&
                compare(obj, inObj, attrName3)==0) return inObj;
        }
        return null;
    }//contain
    
    public Object contain(Object obj, String attrName1, String attrName2, String attrName3, String attrName4) throws NoSuchMethodException
    {
        Iterator iter = this.iterator();
        while (iter.hasNext()) {
            Object inObj = (Object)iter.next();
            if (compare(obj, inObj, attrName1)==0 && 
                compare(obj, inObj, attrName2)==0 &&
                compare(obj, inObj, attrName3)==0 &&
                compare(obj, inObj, attrName4)==0) return inObj;
        }
        return null;
    }//contain
    
    public Object contain(Object obj, String attrName1, String attrName2, String attrName3, String attrName4, String attrName5) throws NoSuchMethodException
    {
        Iterator iter = this.iterator();
        while (iter.hasNext()) {
            Object inObj = (Object)iter.next();
            if (compare(obj, inObj, attrName1)==0 && 
                compare(obj, inObj, attrName2)==0 &&
                compare(obj, inObj, attrName3)==0 &&
                compare(obj, inObj, attrName4)==0 &&
                compare(obj, inObj, attrName5)==0) return inObj;
        }
        return null;
    }//contain
}//SortablyVector
