USE AutoReportGenerator;

CREATE TABLE Projects (
	project_id             INT NOT NULL,
	sw_title               VARCHAR(255) NOT NULL,
	sw_version             VARCHAR(255) NOT NULL,
	description            TEXT NOT NULL,
	changed                TIMESTAMP NOT NULL,
	timestamp              TIMESTAMP NOT NULL,
	PRIMARY KEY (project_id)
);

CREATE TABLE Users (
	project_id             INT NOT NULL,
	user_id                VARCHAR(255) NOT NULL,
	password               VARCHAR(255) NOT NULL,
	initials               VARCHAR(10) NOT NULL,
	e_mail                 VARCHAR(255) NOT NULL,
	roles                  TEXT NOT NULL,
	description            TEXT NOT NULL,
	changed                TIMESTAMP NOT NULL,
	timestamp              TIMESTAMP NOT NULL,
	PRIMARY KEY (project_id, user_id)
);

CREATE TABLE Software_Problem_Report (
	project_id             INT NOT NULL,
	spr_no                 INT NOT NULL,
	scr_no                 INT NULL,
	date                   TIMESTAMP NOT NULL,
	originator             VARCHAR(255) NOT NULL,
	state                  int NOT NULL, /*'1': open '0': close */
	sw_title               VARCHAR(255) NOT NULL,
	sw_version             VARCHAR(255) NOT NULL,
	priority               INT NOT NULL, /* 1: Routine, 2: Urgent, 3: Critical */
	problem_description    TEXT NOT NULL,
	environment            VARCHAR(255) NOT NULL,
	recommended_solution   TEXT NOT NULL,
	srb_decision           TEXT NOT NULL,
	sender_addr            VARCHAR(255) NOT NULL,
	cc_addr                TEXT NOT NULL,
	changed                TIMESTAMP NOT NULL,
	timestamp              TIMESTAMP NOT NULL,
	PRIMARY KEY (project_id, spr_no)
);

CREATE TABLE Software_Change_Request (
	project_id             INT NOT NULL,
	scr_no                 INT NOT NULL,
	smr_no                 INT NOT NULL,
	date                   TIMESTAMP NOT NULL,
	originator             VARCHAR(255) NOT NULL,
	state                  INT NOT NULL, /*'1': open '0': close */
	sw_title               VARCHAR(255) NOT NULL,
	sw_version             VARCHAR(255) NOT NULL,
	priority               INT NOT NULL, /* 1: Routine, 2: Urgent, 3: Critical */
	change_required        TEXT NOT NULL,
	responsible_staff      TEXT NOT NULL,
	start_date             TIMESTAMP NOT NULL,
	end_date               TIMESTAMP NOT NULL,
	manpower               VARCHAR(255) NOT NULL,
	sender_addr            VARCHAR(255) NOT NULL,
	cc_addr                TEXT NOT NULL,
	changed                TIMESTAMP NOT NULL,
	timestamp              TIMESTAMP NOT NULL,
	PRIMARY KEY (project_id, scr_no)
);

CREATE TABLE Software_Modification_Report (
	project_id             INT NOT NULL,
	smr_no                 INT NOT NULL,
	date                   TIMESTAMP NOT NULL,
	originator             VARCHAR(255) NOT NULL,
	sw_title               VARCHAR(255) NOT NULL,
	sw_version             VARCHAR(255) NOT NULL,
	change_implemented     TEXT NOT NULL,
	start_date             TIMESTAMP NOT NULL,
	end_date               TIMESTAMP NOT NULL,
	manpower               VARCHAR(255) NOT NULL,
	sender_addr            VARCHAR(255) NOT NULL,
	cc_addr                TEXT NOT NULL,
	changed                TIMESTAMP NOT NULL,
	timestamp              TIMESTAMP NOT NULL,
	PRIMARY KEY (project_id, smr_no)
);

CREATE TABLE Attachments (
	project_id             INT NOT NULL,
	attachment_id          INT NOT NULL,
	report_no              INT NOT NULL,
	report_type            INT NOT NULL,
	file_name              VARCHAR(255) NOT NULL,
	size                   INT NOT NULL,
	content_type           VARCHAR(255) NOT NULL,
	attachment             MEDIUMBLOB NOT NULL,
	changed                TIMESTAMP NOT NULL,
	timestamp              TIMESTAMP NOT NULL,
	PRIMARY KEY (project_id, attachment_id)
);

CREATE TABLE Sequence_Keys (
	project_id             INT NOT NULL,
	table_name             VARCHAR(255) NOT NULL,
	sequence               INT NOT NULL,
	changed                TIMESTAMP NOT NULL,
	timestamp              TIMESTAMP NOT NULL,
	PRIMARY KEY (project_id,table_name)
);


/*********************************************************************************/
/*                                                                               */
/* Overview of the database and its bindings.                                    */
/* Explanation:                                                                  */
/*    many----many                                                               */
/*    many--->one                                                                */
/*    one<--->one                                                                */
/*                                                                               */
/*  Projects                                                                     */
/*    project_id<---+                                                            */
/*                  |                                                            */
/* Users            |                                                            */
/*    project_id----+                                                            */
/*    user_id       |   Software_Problem_Report                                  */
/*                  +-----project_id                                             */
/*                  |     spr_no<--------------------+                           */
/*                  | +---scr_no                     |                           */
/*                  | |                              |                           */
/*                  | | Software_Change_Request      |                           */
/*                  +-----project_id                 |                           */
/*                  | +-->scr_no<--------------------+                           */
/*                  | +---smr_no                     |                           */
/*                  | |                              |                           */
/*                  | | Software_Modification_Report |                           */
/*                  +-----project_id                 |                           */
/*                  | +-->smr_no<--------------------+                           */
/*                  |                                |                           */
/*                  |                                | Attachments               */
/*                  |                                +---report_no               */
/*                  +------------------------------------project_id              */
/*                                                       report_type             */
/*                                                                               */
/*********************************************************************************/

INSERT INTO Projects (
	project_id,
	sw_title,
	sw_version,
	description,
	changed,
	timestamp
) VALUES (
	1,
	'Default Project',
	'1.0',
	'Default Project for all reports with no projects ID',
	'1980-1-1 00:00:00',
	'1980-1-1 00:00:00'
);

INSERT INTO Sequence_Keys (
	project_id,
	table_name,
	sequence,
	changed,
	timestamp
) VALUES (
	0,
	'Projects',
	0,
	'1980-1-1 00:00:00',
	'1980-1-1 00:00:00'
);
INSERT INTO Sequence_Keys (
	project_id,
	table_name,
	sequence,
	changed,
	timestamp
) VALUES (
	1,
	'Software_Problem_Report',
	0,
	'1980-1-1 00:00:00',
	'1980-1-1 00:00:00'
);
INSERT INTO Sequence_Keys (
	project_id,
	table_name,
	sequence,
	changed,
	timestamp
) VALUES (
	1,
	'Software_Change_Request',
	0,
	'1980-1-1 00:00:00',
	'1980-1-1 00:00:00'
);
INSERT INTO Sequence_Keys (
	project_id,
	table_name,
	sequence,
	changed,
	timestamp
) VALUES (
	1,
	'Software_Modification_Report',
	0,
	'1980-1-1 00:00:00',
	'1980-1-1 00:00:00'
);
INSERT INTO Sequence_Keys (
	project_id,
	table_name,
	sequence,
	changed,
	timestamp
) VALUES (
	1,
	'Attachments',
	0,
	'1980-1-1 00:00:00',
	'1980-1-1 00:00:00'
);



INSERT INTO Users (
	project_id,
	user_id,
	password,
	e_mail,
	initials,
	roles,
	description,
	changed,
	timestamp
) VALUES (
	1,
	'admin',
	'admin',
	' ',
	'Admin',
	'projects,administrator,user,viewer,reviewer',
	'Administrator for  the default project',
	'1980-1-1 00:00:00',
	'1980-1-1 00:00:00'
);
INSERT INTO Users (
	project_id,
	user_id,
	password,
	e_mail,
	initials,
	roles,
	description,
	changed,
	timestamp
) VALUES (
	1,
	'user',
	'user',
	' ',
	'user',
	'user,viewer,reviewer',
	'User of the default project',
	'1980-1-1 00:00:00',
	'1980-1-1 00:00:00'
);
